#
# awk-script to make manual page in either
# latex, html or nroff format
# from C-comment
# C Helge Ritter 1993
#

function DO_TEX_ESCAPES(Token) {
   # NOTE: this function is only called in some places.
   # further tex escapes are made in places
   # FURTHER TEX_ESCAPES I   and 
   # FURTHER TEX_ESCAPES II
   # when implementing additional tex escapes, update code also there!!
   # quote underscores:
   gsub(/_/,"\\@#!~",Token); gsub(/\\@#!~/,"\\_",Token); gsub(/\\\\_/,"\\_",Token);
   # quote hat symbols:
   gsub(/\^/,"\\@#!~",Token); gsub(/\\@#!~/,"\\^",Token); gsub(/\\\\^/,"\\^{}",Token);
   # quote braces {}:
#  gsub(/{/,"\\@#!~",Token); gsub(/\\@#!~/,"\\{",Token); gsub(/\\\\{/,"$\\{$",Token);
#  gsub(/}/,"\\@#!~",Token); gsub(/\\@#!~/,"\\}",Token); gsub(/\\\\}/,"$\\}$",Token);
   if (jj=index(Token,"&")) {   # gsub doesnt work for ampersand!
      Length = length(Token)
      while (jj<=Length) {
         if (substr(Token,jj,1)=="&") {
            # protect ampersand by inserting backslash:
            Token = substr(Token,1,jj-1)"\\"substr(Token,jj)
            jj = jj+2
            Length++
         } else jj++
      }
   }
   return Token
}

function ClearArgList() {
   LAST_ENTRY = 0   # index of last entry in argument description list
                    # 0 = list still empty [starts at 1 ]
   for (k in ARG_INDEX) delete ARG_INDEX[k];
   for (k in ARG_ENTRY) delete ARG_ENTRY[k];
   PARLIST = ""
}

function MakeListForSynopsisLine() {
   if (LANGUAGE=="tex") {
      LIST = BEGIN_BOLD DO_TEX_ESCAPES($3) END_BOLD
      for (k=4; k<=NF && $k != "*/"; k++) LIST = LIST" "DO_TEX_ESCAPES($k)
   } else {
      LIST = BEGIN_BOLD $3 END_BOLD
      for (k=4; k<=NF && $k != "*/"; k++) LIST = LIST" "$k
   }
}

function RequestsArguments(Textline) {
   return (Textline ~ /^\/\*[ ]*@(ARGUMENTS)(:|)[ ]+\*\//);
}

function PrintArgList() {
   # print out collected argument descriptions of most recent
   # function definition(s)  
   print BEGIN_LIST > FILE  # start description list
   for (k=1; k<=LAST_ENTRY; k++) {
      # omit unfilled entries:
      if (TYPE_ENTRY[k]=="" && ARG_ENTRY[k]=="") continue
      if (COMMENT_ENTRY[k]=="") {
         COMMENT_ENTRY[k]="-- not documented in source --"
      }
      if (LANGUAGE=="tex") {
         printf "%s%s%s %s%s%s%s\n",BEGIN_TAG,BEGIN_TT CITE_SPACE,\
         DO_TEX_ESCAPES(TYPE_ENTRY[k]),DO_TEX_ESCAPES(ARG_ENTRY[k]),END_TT,\
         BEGIN_DESCRIPTION,DO_TEX_ESCAPES(substr(COMMENT_ENTRY[k],2)) > FILE
      } else {
         printf "%s%s%s %s%s%s%s\n",BEGIN_TAG,BEGIN_TT CITE_SPACE,\
         TYPE_ENTRY[k],ARG_ENTRY[k],END_TT,\
         BEGIN_DESCRIPTION,substr(COMMENT_ENTRY[k],2) > FILE
      }
   }
   print END_LIST > FILE
}

function AsciifyTexMathSymbols() {
   if ($0 ~ /^(>|)   /) return  # preformatted line: keep!
   # lines not starting with ">   " or "    " are asciified:
   while (jj=index($0,"\\over ")) {
      $0=substr($0,1,jj-1)"/"substr($0,jj+6)
      # resplit current line:
      NF=split($0,Fields);
      for (i=1; i<=NF; i++) $i = Fields[i]
   }
   for (i=1; i<=NF; i++) {
   while (jj=index($i,"\\sum")) $i=substr($i,1,jj-1) substr($i,jj+1)
   while (jj=index($i,"\\prod")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\infty")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\int")) $i=substr($i,1,jj-1)"integral"substr($i,jj+1)
   while (jj=index($i,"\\max")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\min")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\|")) $i=substr($i,1,jj-1)"|"substr($i,jj+1)
   while (jj=index($i,"\\limits")) $i=substr($i,1,jj-1) substr($i,jj+7)
   while (jj=index($i,"\\Delta")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\nabla")) $i=substr($i,1,jj-1)"Grad"substr($i,jj+6)
   while(jj=index($i,"\\over")) $i=substr($i,1,jj-1)"/"substr($i,jj+5)
   while (jj=index($i,"\\left")) $i=substr($i,1,jj-1)substr($i,jj+5)
   while (jj=index($i,"\\right")) $i=substr($i,1,jj-1)substr($i,jj+6)
   while (jj=index($i,"\\in")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\not")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\dot")) $i=substr($i,1,jj-1)"d/dt "substr($i,jj+4)
   while (jj=index($i,"\\vec")) $i=substr($i,1,jj-1)substr($i,jj+4)
   while (jj=index($i,"\\sqrt")) $i=substr($i,1,jj-1)substr($i,jj+1)
   # a subset of greek letters: alpha-phi, lambda, pi, sigma:
   while (jj=index($i,"\\alpha")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\beta")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\gamma")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\delta")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\epsilon")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\sigma")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\pi")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\phi")) $i=substr($i,1,jj-1)substr($i,jj+1)
   while (jj=index($i,"\\lambda")) $i=substr($i,1,jj-1)substr($i,jj+1)

   # replace conditional braces by protected braces:
   while (jj=index($i,"`{")) $i=substr($i,1,jj-1)"\\{"substr($i,jj+2)
   while (jj=index($i,"}'")) {
      if (jj>1 && substr($i,jj-1,1)=="\\") break;
      else $i=substr($i,1,jj-1)"\\}"substr($i,jj+2)
   }
   }
   # resplit current line:
   NF=split($0,Fields);
   for (i=1; i<=NF; i++) $i = Fields[i]
}

function FinishPreviousManpage(NumParts) {
   if (NumParts> 1) printf "  writing %s-MANPAGE to %s (%d parts)\n",LANGUAGE,FILE,NumParts
   else printf "  writing %s-MANPAGE to %s\n",LANGUAGE,FILE
   printf "%sFILE%s\n",BEGIN_H2,END_H2 > FILE
   if (LANGUAGE=="tex") FILEID = DO_TEX_ESCAPES(FILEID);
   printf "%s\n",EMPHASIZE(FILEID) > FILE
   printf "%s%s\n",END_BODY,END_DOCUMENT   > FILE
   if (LANGUAGE!="tex" && LANGUAGE!="html") {
       close(FILE)
       FormatManPage = "eqn -Tlatin1 "FILE"|tbl|nroff -man >"FILE".tmp; cp "FILE" myfile.dat; mv "FILE".tmp "FILE
       system(FormatManPage)
   }
}

function IsManText(Textline) { return (Textline ~ /^\/\*.* @MANTEXT( [^ ]+)*.*/) }  
function IsManEnd(Textline) { return (Textline ~ /^[ ]*@MANEND[ ]*[\*]+\//) }  

function IsManPage(Textline) { 
   if (DO_ALL==1) return (Textline ~ /^\/\*.* @MANPAGE( [^ ]+)*.*/);  
   else return (Textline ~ /^\/\*.* @(|@)MANPAGE( [^ ]+)*.*/) 
}  

function AnnouncesPrototype(Textline) {
   return (Textline ~ /^\/\*[ ]*@(PROTOTYPE|C-SYNTAX)(:|)[ ]+\*\//);
}

function AnnouncesPrototype1(Textline) {
   return (Textline ~ /^[ ]*(@|)(PROTOTYPE|C-SYNTAX)(:|)[ ]+\*\//);
}

function ErrorChecking() {
   # checks, whether field 3 exists and holds current MAN_TITLE:
   if (NF<3)
      printf "  warning in line %d: MANTEXT not followed by name!\n",NR
   if ($3 != FIRST_MANTITLE && $3 != MAN_TITLE &&
       $3 != FIRST_MANTITLE":" && $3 != MAN_TITLE":" ){
      if (FIRST_MANTITLE==MANTITLE)
        printf "  warning in line %d: MANTEXT %s <<<< (%s expected!)\n",NR,$3,MAN_TITLE
      else
        printf "  warning in line %d: MANTEXT %s <<<< (%s or %s expected!)\n",NR,$3,MAN_TITLE,FIRST_MANTITLE
   } 
}

function isManItem(Textline) {
   return (Textline ~ /^(<:)*\-[ ]*(.|\n)*\:.*/) 
}
 
function isArgListEnd(Tok,Suc) {
   return (Tok ~ /.*\,/ || Tok ~ /.*\)[;\{].*/ || (Tok ~ /.*\)/ &&(Suc==";"||Suc~/[{].*/)))
}

function EMPHASIZE(Token) {
   # make dot set \tt-font with delimiters ",.!?;:" if at end of token
   # make accent ` set \em font
   # additionally, an unpaired bracket at end of token is delimiter too 
   if (Token == "...") return Token  # leave these as a special case
   TokenStart = substr(Token,1,1)
   if (TokenStart=="." || TokenStart=="`") {
      TokenLen = length(Token)
      TokenEnd = substr(Token,TokenLen,1)
      if (TokenStart==".") Delimiters=",.!?:"   
      # DO NOT include  ';' because .<:xxx:> is translated into
      #  .&lt;:xxx:&gt;   (using the html escapes for < and >!) 
      else Delimiters=""
      if (index(Delimiters,TokenEnd) && TokenLen>2) {
         TokenLen=TokenLen-2   # excludes delimiter from emph
      } else if (TokenEnd==")" && !index(Token,"(")) {
         TokenLen=TokenLen-2   # excludes delimiter from emph
      } else {
         TokenEnd=""
         TokenLen=TokenLen-1
      }
   }
   if (TokenStart==".") {
      if (substr(Token,2,1)==".") {
         Token =  BEGIN_TT substr(Token,2,TokenLen) END_TT TokenEnd
      } else {
         Token = BEGIN_TT TT_SPACE substr(Token,2,TokenLen) END_TT TokenEnd
      }
   } else if (TokenStart=="`") {
      # make ` set \em-font
      if (substr(Token,2,1)=="`") {
         Token =  BEGIN_CITE substr(Token,2,TokenLen) END_CITE TokenEnd
      } else {
         Token = BEGIN_CITE CITE_SPACE substr(Token,2,TokenLen) END_CITE TokenEnd
      }
   }
   return Token
} 

BEGIN {

   FIRSTTIME = 1;
   MAXARGS = 50   # no more than 50 args per function!
   INSERTED_C_SYNTAX = 0
   EXPECT_FUNCTION_DEF = 1
   PARSE_FUNC_ARGS = 2
   PARSE_ARG_COMMENT = 3
   PARSE_MAIN_COMMENT = 4
   STATUS = 0
   ACCEPT_NEXT = 0
   FTYPE  = ""
   INDENTED_MARGIN = 0
   MAN_TITLE = ""
   NOHEADER =1
   MAN_NAME = "nst"
   MAN_VERSION = "3.0"
   if (MAN_DIR == "") MAN_DIR = "."
   FOOTNOTE = "$^+$"
   CVERBATIM = 0
   EXPECT_CONTINUATION=0
   for (i=1; i<=9; i++) VarVal[i]="/vol/nst/man/html3"  # defaults 
}

#
# language specific definitions:
#
 
(NR==1) {
   if (index(PATTERN,"+h "))  NOHEADER = 0  # add header for standalone latex src
   if (index(PATTERN,"-M "))  ONLY_DEP_INFO = 1  # make only dependency info
   if (index(PATTERN,"-all ")) DO_ALL = 1
   if (index(PATTERN,"-All ")) DO_ALL = 2
   if (index(PATTERN,"-ind ")) INDENTED_MARGIN = 1
   if (LANGUAGE != "tex" && LANGUAGE != "html") LANGUAGE = "nroff"
   if (LANGUAGE == "tex") {
   if (NOHEADER) {
      # the following line is for used by other tools to 
      # recognize headerless mma latex manpages in a directory
      # thus, do not change this line!! 
      HEADERCOMMENT  = "%% latex include file generated by mma %%\n"
      BEGIN_DOCUMENT = ""
      BEGIN_HEADER   = ""
      END_BODY       = "\n%% end of manual page\n"
   } else {
      HEADERCOMMENT  = "%% manual page source format generated by mma %%\n"
      BEGIN_DOCUMENT = "\\documentstyle[11pt]{article}\n\\textwidth16cm\\textheight24cm\n"\
                       "\\hoffset-2.5cm\\voffset-2cm\n\\parindent0pt\\itemindent2cm\\itemsep0pt\n"
      BEGIN_HEADER   = "\\parskip0.15\\baselineskip\\begin{document}{\\itemindent2cm\\itemsep0pt\n"
      END_BODY       = "}\\end{document}"
   }
   END_DOCUMENT   = ""
   END_HEADER     = ""
   BEGIN_BODY     = ""
   BEGIN_TITLE    = "{\\noindent\\bf "
   END_TITLE      = " }\\par"
   BEGIN_H2       = "\\par\\vspace{\\baselineskip}\\noindent{\\Large\\bf "
   END_H2         = "}\\par\\vspace{0.3\\baselineskip}\\noindent"
   BEGIN_H3       = "\\par\\vspace{0.7\\baselineskip}\\noindent{\\large\\bf "
   END_H3         = "}\\par\\vspace{0.2\\baselineskip}\\noindent"
   BEGIN_LIST ="\\begin{description}\n"
   BEGIN_TAG  = "\\item["
   BEGIN_DESCRIPTION = "]"
   END_LIST   ="\\end{description}\n\\noindent"
   BEGIN_BOLD = "{\\bf "
   END_BOLD   = "}"
   # old: BEGIN_PAR  = "\\par\\noindent\n"
   BEGIN_PAR  = "\n\\par\\noindent "
   BEGIN_CITE = "{\\em "
   CITE_SPACE = " "
   END_CITE   = "}"
   BEGIN_TT = "{\\tt "
   TT_SPACE = " "
   END_TT   = "}"
   BEGIN_PRE  = "{\\tt\\begin{verbatim}"
   END_PRE    = "\\end{verbatim}}"
   LT_CHAR    = "<"
   } else if (LANGUAGE == "html") {
   HEADERCOMMENT  = "<!-- manual page source format generated by mmp -->\n"
   BEGIN_DOCUMENT = "<HTML>"
   END_DOCUMENT   = "</HTML>"
   BEGIN_HEADER   = "\n"
   END_HEADER     = ""
   BEGIN_BODY     = "<BODY>"
   END_BODY       = "</BODY>"
   BEGIN_TITLE    = "<TITLE>"
   END_TITLE      = "</TITLE>"
   BEGIN_H2       = "<H2>"
   END_H2         = "</H2>"
   BEGIN_H3       = "<H3>"
   END_H3         = "</H3>"
   BEGIN_LIST ="<DL>\n"
   BEGIN_TAG  = "<DT>"
   BEGIN_DESCRIPTION = "<DD>"
   END_LIST   ="</DL>\n"
   BEGIN_BOLD = "<B>"
   END_BOLD   = "</B>"
   BEGIN_PAR  = "<P>\n"     
   BEGIN_CITE = "<CITE>"
   END_CITE   = "</CITE>"
   BEGIN_TT = "<TT>"
   END_TT   = "</TT>"
   BEGIN_PRE  = "<PRE>\n"
   END_PRE    = "</PRE>\n"
   LT_CHAR    = "&lt;"
   } else {  # assume nroff - format
   HEADERCOMMENT  = ""
   BEGIN_DOCUMENT = ""
   END_DOCUMENT   = ""
   BEGIN_HEADER   = ""
   END_HEADER     = ""
   BEGIN_BODY     = ""
   END_BODY       = ".ex"
   BEGIN_TITLE    = ".TH "
   END_TITLE      = ""
   BEGIN_H2       = ".SH "
   END_H2         = ""
   BEGIN_H3       = ".SH "
   END_H3         = ""
   BEGIN_LIST =""
#  BEGIN_TAG  = "\n.PP\n.TP 20\n"  
   BEGIN_TAG  = ".TP 20\n"  
   BEGIN_DESCRIPTION = "\n"  
   END_LIST   =""
   BEGIN_BOLD = "\\fI"
   END_BOLD   = "\\fR"
   BEGIN_PAR  = "\n.PP"  #was "\n.PP"
   BEGIN_CITE = "\\fI"
   END_CITE   = "\\fR"
   BEGIN_TT   = "\\fI"
   END_TT     = "\\fR"
   BEGIN_PRE  = ".nf"
   END_PRE    = ".fi"  # ".fi\n.sp 1"
   LT_CHAR    = "<"
   }
   PATTERN = " "PATTERN"@@@@@"  # now we can require surrounding blanks
   # the @@@'s make a match with the file name (is last in PATTERN) unlikely
}

# this line skips everything up to the next continuation: */

(STATUS == EXPECT_FUNCTION_DEF) {
   # swallow blank lines:
   if ($0 ~ /^[ ]*$/) next;
}

# this line ensures proper termination of the current man page
# after a MAIN_COMMENT (HR Jun-6-2000):

(STATUS == PARSE_MAIN_COMMENT) {
   if ($0 ~ /^.*\*\/[ ]*$/) {
      STATUS = 0
      next;
   }
}

(EXPECT_CONTINUATION) {
   if (IsManText($0)) {  
      ErrorChecking()
      EXPECT_CONTINUATION = 0
      STATUS = PARSE_MAIN_COMMENT
      TEXT_PART++
   } else if (AnnouncesPrototype($0)) {
      # expect to parse prototype next: 
      STATUS              = EXPECT_FUNCTION_DEF
      ACCEPT_NEXT         = 0
      EXPECT_CONTINUATION = 0
      PRINT_ARGS          = 0
      ClearArgList();
      next
   } else if (IsManPage($0)) {
      # start a new man page:
      EXPECT_CONTINUATION = 0
      STATUS = 0
   } else next  # ignore line 
}
 
################# Handling of HTMLDEF- Section: ################
# 
#
########################################################################

{
if ( $0 ~ /^(|\/[\*]+ ) *@HTMLDEF(S|S:) *$/) {
   HTMLDEFSTRUE = 1;  # start HTMLDEFS - section.
}
if (HTMLDEFSTRUE && $0 ~ /^ *$/) {
   HTMLDEFSTRUE = 0; next; # blank line ends HTMLDEFS-section
}
if (HTMLDEFSTRUE) {
   if ($2 ~ /\$[1-9]/) {
       # set variable VarVal[i] to content of $1:
       kk = substr($2,2,1) # this is the digit
       VarVal[kk] = $1
       if (LANGUAGE=="html") printf "  HTML-define: $%d = \"%s\"\n",kk,$1
   }
   next
}
}

#####################################################################
#   
# pick out lines  /* @MANDEF ...   
#
#####################################################################


($0 ~ /^\/\*[ ]+@MANDEF[ ]+[.]*/) {

   # process comment line of form
   # /* @MANDEF ManName ManVersion ManDir  */
   # to set ManName, ManVersion and ManDir values:
   
   if (NF!=6) {
      print "Error in line "NR" of file "FILEID
      print "(missing arguments of @MANDEF)"
      exit
   }
   MAN_NAME = $3
   MAN_VERSION = $4 
   MAN_DIR = $5
   if (!(ONLY_DEP_INFO)) {
     print "making manual page for "MAN_NAME
     print "version is:            "MAN_VERSION
     print "directory is:          "MAN_DIR
   }
} 

#####################################################################
#   
# pick out lines  /* @MANPAGE pattern ...:  
#
#####################################################################

($1=="/*" && ($2=="@MANPAGE" || $2=="@@MANPAGE")\
          && (ACCEPT_NEXT || (index(PATTERN,$3" ") + DO_ALL >0))) {
   if (DO_ALL==1 && $2=="@@MANPAGE") next;
   #  @@MANPAGES only if NOT -all was specified 

   if (ONLY_DEP_INFO) {
     if (substr(SUFFIX,1,1)=="-") { dep_suf=substr(SUFFIX,2) ; }
     else { dep_suf=SUFFIX ; }
     dep_file="$(RELOBJDIRPATH)"FILEID; #gsub("^.*/","",dep_file);
     printf "%s.%s   : %s\n", $3,dep_suf,dep_file;
     printf "%s.html: %s\n", $3,dep_file;
     printf "%s.tex : %s\n", $3,dep_file;
     dep_file_list=dep_file_list" "$3;
#         if (substr(SUFFIX,1,1)=="-") {
#            printf "%s.%s : %s\n",$3,substr(SUFFIX,2),FILEID
#         } else {
#            printf "%s.%s : %s\n",$3,SUFFIX,FILEID
#         }
       next;
   }

   if (MAN_TITLE != "" && !ACCEPT_NEXT) {
      if (index(PATTERN,FIRST_MANTITLE" ") + DO_ALL) {
         FinishPreviousManpage(TEXT_PART)
         # finish & ship out rest of previous manpage 
      }   
   }

   MAN_TITLE = $3
   while (jj=index(MAN_TITLE,"\\_")) {
       MAN_TITLE=substr(MAN_TITLE,jj-1)substr(MAN_TITLE,jj+1)
   }
   if (ACCEPT_NEXT) {
      suff=substr(SUFFIX,2);
      command="cd "MAN_DIR"; rm -f "MAN_TITLE"."suff;
      command=command" ; ln -s "FIRST_MANTITLE"."suff" "MAN_TITLE"."suff;
      system(command)
      printf "- appending text for %s (sym-linking to %s.%s)\n", \
        MAN_TITLE,FIRST_MANTITLE,suff
   } else {
      printf "- making MANPAGE for %s\n", MAN_TITLE
      FILE = MAN_DIR"/"MAN_TITLE"."substr(SUFFIX,2)   # cut off "-" from flag
      FIRST_MANTITLE = MAN_TITLE 
      TEXT_PART = 1    
   }
   # if found, there are two subcases:
   # a) the comment line ends with '*/'. 
   #    In this case, expect to parse a function name next.
   if ($NF == "*/") {
      PRINT_ARGS      = 1
      PRINT_PROTOTYPE = 1
      if (STATUS==EXPECT_FUNCTION_DEF) {  # new if: dealing with multiple defs (HR)
          # was already expecting function name, found, however, 
          # another @MANPAGE directive which shall be included 
          # in current page. Since header is already there, 
          # print only a further synopsis line:
          MakeListForSynopsisLine();
          printf "%s%s\n", LIST,BEGIN_PAR > FILE
          next
      }
      STATUS=EXPECT_FUNCTION_DEF  # first @MANPAGE directive changes status
   } else { 
      # since no end of comment after @MANPAGE, further manpage text
      # apparently follows directly:
      # 
      MakeListForSynopsisLine();
      if (FIRSTTIME) {
         FIRSTTIME = 0;
         printf "%s", HEADERCOMMENT > FILE
         printf "%s%s",BEGIN_DOCUMENT,BEGIN_HEADER > FILE
      }
#     printf "%s%s\n", LIST,BEGIN_PAR > FILE
      printf "%s%s%s%s\n", BEGIN_TITLE,LIST,END_TITLE,BEGIN_PAR > FILE
      STATUS=PARSE_MAIN_COMMENT
      FIRST_PART_OF_MANPAGE = 1
      PRINT_PROTOTYPE       = 0

   }

   ClearArgList()
   
   # b) else, the comment continues. In this case, the
   #    comment will be the main description of a whole program:
	
   j = 0 

#####################################################################
#   
#  open HTML-File and write header:
#
#####################################################################

   if (STATUS == EXPECT_FUNCTION_DEF) {
      MakeListForSynopsisLine()
      printf "%s", HEADERCOMMENT > FILE
      printf "%s%s",BEGIN_DOCUMENT,BEGIN_HEADER > FILE
      if (LANGUAGE=="tex") {  # insert additional index-command for tex:
         printf "%sman %s%s\n",BEGIN_TITLE,MAN_NAME,END_TITLE > FILE
         printf "%s%s\n",END_HEADER,BEGIN_BODY  > FILE
         printf "%sNAME%s\n\\index{%s}%s%s\n",BEGIN_H2,END_H2,DO_TEX_ESCAPES(MAN_TITLE),LIST,BEGIN_PAR > FILE
      } else if (LANGUAGE=="html") {    # HTML
         printf "%s%s (%s man page)%s\n",BEGIN_TITLE,MAN_TITLE,MAN_NAME,END_TITLE > FILE
         printf "%s%s\n",END_HEADER,BEGIN_BODY  > FILE
         printf "%sNAME%s\n%s%s\n",BEGIN_H2,END_H2,LIST,BEGIN_PAR > FILE
      } else {
         printf "%s%s (%s man page)%s\n",BEGIN_TITLE,MAN_TITLE,MAN_NAME,END_TITLE > FILE
         printf "%s%s\n",END_HEADER,BEGIN_BODY  > FILE
         printf "%sNAME%s\n%s",BEGIN_H2,END_H2,LIST > FILE
      }
      ACCEPT_NEXT = 1;  #  temporarily allow any routine name for 
      # immediately subsequent MANPAGEs. This allows to print 
      # multi-routine manpages by specifying just the first routine 
      next;
   } 
}


#####################################################################
#   
#  the following is applied to each line for which mma must 
#  generate output:
#
#####################################################################

(STATUS) {  # the following is tried on each line for which mmh has to
            # produce output:

ACCEPT_NEXT = 0  # ensure restriction to routine names in list 

#####################################################################
#   
#  interprete some tex-commands; ignore others.
#
#####################################################################

if (LANGUAGE == "html") {
   AsciifyTexMathSymbols()
   # ge and lt are more special: 
   for (i=1; i<=NF; i++) {
      if (jj=index($i,"\\le")) $i=substr($i,1,jj-1)"=<"substr($i,jj+3)
      if (jj=index($i,"\\ge")) $i=substr($i,1,jj-1)">="substr($i,jj+3)
   }
} else if (LANGUAGE == "nroff") AsciifyTexMathSymbols();

################# Handling of embedded HTML-Definitions: ################
# 
#  Token sequences <:TokenSequence:Document> on each line are replaced 
#  by TokenSequence alone and cause an entry in an auxiliary file that 
#  defines a HTML-
#  Link for each occurrence of TokenSequence. The link will lead to document 
#  Document, if Document is specified, to the default document 
# TokenSequenceWithoutSpaces.html otherwise. If Document is a path 
# (i.e. ends with a "/", it will be prepended to the default 
# document name.
# All occurrences of TokenSequence in the created Man Page will be made
# into HTML-links. To restrict the matches, one can include the right
# and or left word on the line in which the <:..:> occurs by omitting
# spaces between <:..:> and the neighbor(s) to be included in the 
# matching context.
#
########################################################################
if ( $0 ~ /^<:([A-Z0-9]|\\")[A-Z a-z_0-9>/\.:\\"]*>[ ]*:[ ]*$/ ||\
     $0 ~ /^>:([A-Z0-9]|\\")[A-Z a-z_0-9</\.:\\"]*<[ ]*:[ ]*$/) SECTIONTRUE = 1
else SECTIONTRUE=0;
if ($0 ~ /^<:([A-Z 0-9]|\\")(.)*>::[ ]*$/ ||\
    $0 ~ /^>:([A-Z 0-9]|\\")(.)*<::[ ]*$/ ) SUBSECTIONTRUE = 1
else SUBSECTIONTRUE=0; 

############# Filtering: all text must here through: ##################


########################################################################
#   
#  deal with starred comments and remove formatting protection char '>' 
#
########################################################################

if (STATUS == PARSE_MAIN_COMMENT) {  
  if (INDENTED_MARGIN) {
     # a 3 char margin is removed in every comment
     if ((substr($0,1,3)=="   ")) $0=substr($0,4);
  } 
  if (substr($0,1,4) == ">   ") $0=substr($0,2);
  # a '>' in the first column, followed by at least 3 spaces, is
  # taken as a protector mark for indentation and is removed.
}


########################################################################
#   
#  do language specific escapes: 
#
########################################################################

Left=""
# escape all special chars:

if (LANGUAGE=="nroff") {
  # deal with char sequences such as "\n" :
  jjprev = 0
  while ((jj=match($0,/\\[0-9a-zA-Z]/)) && jj>jjprev) {
       $0=substr($0,0,jj-1)"\\"substr($0,jj)
       jjprev = jj+1 
  }
}

if (LANGUAGE=="html") {

# umlaute in html:

gsub("&","&amp;",$0);
# since gsub doesn not seem to work properly in the presence of '\':
while (jj=index($0,"\\%")) {
$0 = substr($0,1,jj-1)"%"substr($0,jj+2)
}
while (jj=index($0,"\\\"u")) {
$0 = substr($0,1,jj-1)"&uuml;"substr($0,jj+3)
}
while (jj=index($0,"\\\"o")) {
$0 = substr($0,1,jj-1)"&ouml;"substr($0,jj+3)
}
while (jj=index($0,"\\\"a")) {
$0 = substr($0,1,jj-1)"&auml;"substr($0,jj+3)
}
while (jj=index($0,"\\\"A")) {
$0 = substr($0,1,jj-1)"&Auml;"substr($0,jj+3)
}
while (jj=index($0,"\\\"O")) {
$0 = substr($0,1,jj-1)"&Ouml;"substr($0,jj+3)
}
while (jj=index($0,"\\\"U")) {
$0 = substr($0,1,jj-1)"&Uuml;"substr($0,jj+3)
}
}
# the "<" and ">" must be converted temporarily even if 
# LANGUAGE != "html", since the code below works only for
# the html-replacements:

while (jj=index($0,"<")) {
$0 = substr($0,1,jj-1)"&lt;"substr($0,jj+1)
}
while (jj=index($0,">")) {
$0 = substr($0,1,jj-1)"&gt;"substr($0,jj+1)
}
# make initial dot set em-font:
Indent=""
Length=length($0);
# find leftmost nonspace:
for (i=1; i<=length($0); i++) {
   if (substr($0,i,1)!=" ") break;
   Indent=Indent" "
}
NewLine=""
for (i=1; i<=NF; i++) {
  if (!CVERBATIM) {
     NewLine = NewLine" "EMPHASIZE($i)
  } else NewLine=NewLine" "$i
}
$0 = NewLine

####################  Filtering done #####################

NF=split($0,Fields);
for (i=1; i<=NF; i++) $i = Fields[i]
# leading spaces in $0 are lost due to assigning $i=Fields[i]
# restore them:
$0 = Indent $0  # restore leading spaces (are lost after split!)
Left=""
for (i=1; i<=NF; i++) {
if (!Left) {  # look for delimiter starting def of a link:
   idx1 = index($i,"&lt;:")
   idx2 = index($i,"&gt;:")
   # take leftmost occurrence as relevant:
   if ( (idx1>0) && ((idx1<idx2) || idx2==0) )  {
      Left=i; iLeft=idx1; Delim ="&gt;";
      LeftContext = substr($(Left),1,iLeft-1);
   } else if (idx2>0) {
      Left=i; iLeft=idx2; Delim="&lt;";
      LeftContext = substr($(Left),1,iLeft-1);
   }
}  

if (Left && ((Delim=="&gt;" && $i ~ /:[^:]*(&gt;)/)||\
             (Delim=="&lt;" && $i ~ /:[^:]*(&lt;)/)) ) {
  idx1=1;
  RestStr = substr($i,idx1+1)
  do {  # find idx1 of leftmost matching ":...>" substring: 
	 jj = index(RestStr,":");
         kk = index(RestStr,Delim)
         if (jj && kk && (kk>jj)) {
            idx2 = idx1 + kk + 3
            idx1 += jj 
            RestStr = substr(RestStr,jj+1)
         } else break;
  } while (1)
  # now idx1 = pos of ':', idx2 = last char of Delim in $i
  # printf "################\nfound (Line %d, token %d at [%d,%d]:\"%s\"\n%s\n",\
  #         NR,i,idx1,idx2,substr($i,idx1,idx2+1-idx1),$i
  if (!idx2) continue;
  RightContext = substr($i,idx2+1); 
  # printf "R=%s\n",RightContext
  if (i==Left) Key=substr($i,iLeft+2+3,idx1-iLeft-2-3)
  else Key = substr($(Left),iLeft+2+3); 
  if (!Key) LeftContext = LeftContext" "
  VarVal[0] = Key  # VarVal[0] == $0 
  if (Key) {
	 iNumKeyToks=1
	 VarVal[0]=Key  # VarVal collects Tokens in Key
  }
  for (jj=Left+1; jj<i; jj++) {
	 Key = Key" "$(jj); 
         VarVal[0]=VarVal[0] $(jj)
         ++iNumKeyToks
         # KeyTok[++iNumKeyToks]=$(jj)
  }
  if (i>Left) {
	 if (idx1>1) {
                ++iNumKeyToks
		# KeyTok[++iNumKeyToks]=substr($i,1,idx1-1)
		Key = Key" "substr($i,1,idx1-1) 
		VarVal[0] = VarVal[0] substr($i,1,idx1-1)
	 }
	 else RightContext=" "RightContext
  }
  # extract document name and/or set default name:
  HtmlDoc = substr($i,idx1+1,idx2-idx1-1-3) # idx2 by 3 larger
  # in HtmlDoc substitute $i-Variables:
  Suffix = ""
  if (Delim=="&gt;") {
     if (Key) Suffix = ".html"  # for <..> - case
  }
  if (HtmlDoc) {
     # substitute any token '$k' in HtmlDoc by VarVal[k]:
     Len = length(HtmlDoc)
     for (k=1; k<Len; k++) {
        if (substr(HtmlDoc,k,1)=="$") {
           jj = substr(HtmlDoc,k+1,1)
           if (0<=jj && jj<=9) {
              HtmlDoc = substr(HtmlDoc,1,k-1)VarVal[jj]substr(HtmlDoc,k+2)
              Len = length(HtmlDoc)
           }
        }
     }
     # append suffix ".html", if there is none and first char!='#'.
     if ((!index(HtmlDoc,".")) && index(HtmlDoc,"#")!=1 &&Suffix) {
             HtmlDoc=HtmlDoc Suffix
     }
  } else HtmlDoc=VarVal[0]".html"
  # Reassemble $0:
  NewLine = ""
  for (jj=1; jj<Left; jj++) {
     NewLine = NewLine $(jj) " "
  }
  NewLine = NewLine LeftContext
  if (LANGUAGE == "html") {
     if (Key) {
        if (Delim=="&gt;") {
           if (!HtmlDoc) HtmlDoc=VarVal[0]".html" # default
           NewLine = NewLine  "<A\nHREF=\""HtmlDoc"\">"Key"</A>"RightContext
        } else {
           if (!HtmlDoc) HtmlDoc=VarVal[0]  # default
           NewLine = NewLine "<A\nNAME=\""HtmlDoc"\">"Key"</A>"RightContext
        }
     } else {
     # if key field is empty, and first char of path is '!'
     # the rest content is sent as HTML-cmd <rest>. Example:
     #  <::!cite>this text<::!/cite>   sends 
     #  <cite>this text</cite>
   
     if (substr(HtmlDoc,1,1)=="!") {
         # restore original form:
         if (LANGUAGE=="tex") {
            gsub("&lt;","$\\langle$",HtmlDoc)
            gsub("&gt;","$\\rangle$",HtmlDoc)
         } else {
            gsub("&lt;","<",HtmlDoc)
            gsub("&gt;",">",HtmlDoc)
         }
         printf "#####restored cmd: <%s>\n",HtmlDoc
         NewLine = NewLine "<"substr(HtmlDoc,2)">" RightContext
     } else {
     # if key field is empty, the specification is interpreted as
     # an image. The name string is broken into fields delimited
     # by ';'. A >..< - image is not anchored, a <..> image is.:
     # subcommands can be appended, separated by ';'s.
     NumCmds=split(HtmlDoc,CmdList,";"); NumCmds1=NumCmds;
     if (NumCmds<3)  Name=CmdList[1]; else Name=CmdList[NumCmds--]
     CmdStr = ""
     for (kk=2; kk<=NumCmds; kk++) CmdStr = CmdStr" "CmdList[kk]
     if (Delim=="&lt;") {
        # >::file< is interpreted as insertion of a named image,
        # name being the last argument
        NewLine = NewLine "<A NAME=\""Name"\"><IMG SRC=\""\
                  CmdList[1]"\" ALIGN="CmdStr"></A>"RightContext
     }
     if (Delim=="&gt;") {
        # <::file;subcmds..;dest> is interpreted as an anchored
        # image file, anchored to its last argument dest.
        if (NumCmds1<3) {
            # name = same as image file, replace image-suffix by "html":
           if (kk=index(Name,".")) Name=substr(Name,1,kk)"html"
        } else {
           # name = last arg. If it has suffix, keep it.
           # otherwise, append default suffix ".html"
           if (!(kk=index(Name,"."))) Name=Name".html"
        }
        NewLine = NewLine "<A HREF=\""Name"\"><IMG SRC=\""\
                  CmdList[1]"\" ALIGN="CmdStr"></A>"RightContext
     }
     }
     } 
  } else {  # non-HTML-case:
     NewLine = NewLine Key RightContext  # suppress all HTML-adornments in output
  }
  for (jj=i+1; jj<=NF; jj++) NewLine = NewLine" "$(jj)
  # Reassemble $i's:
  NF=split(NewLine,Fields);
  # printf "NF=%d %s\n",NF,NewLine
  for (jj=1; jj<=NF; jj++) {
    # printf "i=%d %s\n",jj,Fields[jj]
    $(jj) = Fields[jj]  # NEWINSERTED
  }
  # leading spaces in $0 are lost due to assigning $i=Fields[i]
  # restore them:
  $0 = Indent NewLine  # restore leading spaces (are lost after split!)
  idx1 = 1;  # OBSOLETE?
  # anchor must embrace Left..former i
  Left=""; # clear for next token
  # Result at this point:
  # LeftContext, RightContext : enclosing contexts
  # Key     : token sequence with spaces
  # VarVal[0] : token sequence without spaces
  # HtmlDoc : document or link name
  # Delim   : ">", "<" for outgoing/ingoing link
  # Edit into HTML-Line:
  # so far test output
  # printf "Html-Def (%s) in Line %d: Doc=\"%s\"\n",Delim,NR,HtmlDoc
  # printf "L=\"%s\" K=\"%s\" R=\"%s\"\n",LeftContext,Key,RightContext
  NewLine=""
  HtmlDoc=""
} /* if */
} /* for */

if (LANGUAGE != "html") {
   if (LANGUAGE=="nroff") {
      for (i=1; i<=NF; i++) {
         if (jj=index($i,"\\break")) $i=substr($i,1,jj-1)"\n"substr($i,jj+6)
         if (jj=index($i,"\\le")) $i=substr($i,1,jj-1)"=<"substr($i,jj+3)
         if (jj=index($i,"\\ge")) $i=substr($i,1,jj-1)">="substr($i,jj+3)
      }
   } 
   if (LANGUAGE != "tex" || (CVERBATIM || (match($0,/^   /) && PREVIOUS~/^[ ]*$/))) {
   # not tex or within comment: do backsubstitution to "<" and ">":
      while (jj=index($0,"&amp;")) { $0 = substr($0,1,jj-1)"&"substr($0,jj+5) }
      while (jj=index($0,"&lt;")) { $0 = substr($0,1,jj-1)"<"substr($0,jj+4) }
      while (jj=index($0,"&gt;")) { $0 = substr($0,1,jj-1)">"substr($0,jj+4) }
   } else {
      # tex and outside comment: resubstitute &amp; &lt; and &gt;
      while (jj=index($0,"&amp;")) {
         $0 = substr($0,1,jj-1)"\\&"substr($0,jj+5) 
      }
      while (jj=index($0,"&lt;")) {
         $0 = substr($0,1,jj-1)"{\\tt<}"substr($0,jj+4) 
      }
      while (jj=index($0,"&gt;")) { 
            $0 = substr($0,1,jj-1)"{\\tt>}"substr($0,jj+4) 
      }
   }
} else {  # html:
   for (i=1; i<=NF; i++) {
      if (jj=index($i,"\\break")) $i=substr($i,1,jj-1)"<BR>"substr($i,jj+6)
   }
   if (0 && !(CVERBATIM || (match($0,/^   /) && PREVIOUS~/^[ ]*$/))) {
      while (jj=index($0,"=&lt;")) {
         $0 = substr($0,1,jj-1)"&le;"substr($0,jj+5) 
      }
      while (jj=index($0,"&gt;=")) { 
         $0 = substr($0,1,jj-1)"&ge;"substr($0,jj+5) 
      }
   }
} 

################# End Handling of embedded HTML-Definitions: ################
  
if (STATUS==EXPECT_FUNCTION_DEF && IsManText($0)) {
      # if instead of an expected function definition there
      # is a @MANTEXT directive, switch expectation to
      # handling of MAIN_COMMENT:
      ErrorChecking()
      EXPECT_CONTINUATION = 0
      STATUS = PARSE_MAIN_COMMENT
      FIRST_PART_OF_MANPAGE = 0
      PRINT_PROTOTYPE       = 0
      TEXT_PART++
}

if (STATUS==EXPECT_FUNCTION_DEF && AnnouncesPrototype($0)) {
   # this is an almost redundant @PROTOTYPE declaration
   PRINT_ARGS          = 0
   # (expecting function def already! 
   # Ignore it:
   next
}

##########################################################################
#
#   If a function definition is encountered, process it here:
#
##########################################################################

if (STATUS==EXPECT_FUNCTION_DEF )  {
   # parse the definition of a function,

   if ($0 !~ /^[ ]*$/) {  # for non-blank lines do:
       for (i=1; i<=NF; i++) { 
          # token i that matches *(* indicates beginning of
          # parameter list. The part before the ( is the name:
    	  if ($i ~ /[.]*\([.]*/) { 
    	     STATUS=PARSE_FUNC_ARGS;  #  change status to parse
    							#  argument list next.
    	     VOID_ARGS=index($i,"(void)")
    	     VOID_END = !index($i,"(void);") 
    	     FNAME=substr($i,0,index($i,"(")-1); #  singles out name.
                 if (length($i)>1+length(FNAME)) {
         	        NEXT_TOKEN_INDEX = i
                 } else NEXT_TOKEN_INDEX = i+1 #  next token behind "name( " 
                 if (NEXT_TOKEN_INDEX>NF) NEXT_TOKEN_INDEX=1
                 break   # function name has been found.
     	  } else if ($i != "extern") {
              # type keyword "extern" is ``swallowed''.
              if (FTYPE != "") FTYPE = FTYPE" "$i
              else FTYPE = $i 
          }
       }
   }  
}

if (STATUS==PARSE_FUNC_ARGS) {

# now proceed to parse the argument list of the function.
# Build a list of triples TYPE[j], ARG[j], COMMENT[j],
# one for each argument parameter j that is found.

   for (i=1; i<=NF; i++) {
      if ($i=="/*" || $i==",/*" || CONTINUE_COMMENT) {
         if (!CONTINUE_COMMENT) {
 	        # after token $i, there is a comment that informs
	        # about the preceding argument $i-1 of type $i-2:
	        TYPE[j]="" 
	        # build type specifier:
	        T = $(NEXT_TOKEN_INDEX)  # $(i-2) 
	        if (T ~ /\(/ && substr(T,0,index(T,"(")-1)==FNAME) {	    
	           # deal with special case that first type specifier 
		       # is directly adjacent to "name(":   
	           pos=index(T,"(")
	           TYPE[j]=substr(T,pos+1,length(T)-pos)
	           NEXT_TOKEN_INDEX++
	        }
	        for (k=NEXT_TOKEN_INDEX; k<=i-2; k++) {
	           TYPE[j]=TYPE[j]" "$k 
	        }
	        ARG[j]=$(i-1)  # this is the argument token
	        if (substr(ARG[j],length(ARG[j]),1)==",")
	           # discard possible trailing comma:
	           ARG[j]=substr(ARG[j],0,length(ARG[j])-1)
         } else i=0  # will be set to 1 in next statement   	     	
	     while ($++i != "*/" && $i != "*/," && $i != "*/)" && $i != "*/);") {
	        # find end of comment belonging to current argument
            # make dot set \tt-font, but protect combination ".*/" at end of comment: 
            if (substr($i,1,1)==".") {
                   $i = EMPHASIZE($i) # BEGIN_CITE substr($i,2) END_CITE 
            }
            if (split($i,PARTS,/[*][/]/)>1) { 
                   COMMENT[j]=COMMENT[j]" "PARTS[1];  
                   i++;
                   break;
            } else { 
	           COMMENT[j]=COMMENT[j]" "$i
	           if (i==NF) {
		          CONTINUE_COMMENT=1  # indicator to continue comment
		          next
		       }
            }
	     }  
         # now entire comment text for argument j 
	     # is collected in comment[j]
	     CONTINUE_COMMENT=0
         NEXT_TOKEN_INDEX = i+1
	     if (NEXT_TOKEN_INDEX>NF) NEXT_TOKEN_INDEX=1
	     j++
	  } else if (isArgListEnd($i,$(i+1)) && !CONTINUE_COMMENT && !VOID_ARGS) { 
	     # treat case when no comment is present:
	     if ((i<NF && ($(i+1) !~ /\/\*/)) || i==NF) {
	        COMMENT[j] = "" 
            # "-- not documented in source --"
	        if ($i==",") {
		       ARG[j] = $(i-1)
		       ARGPOS = i-1
		    } else if (index($i,",")) {
	           COMMENT[j] = ""  # newly inserted 
		       ARG[j]=substr($i,1,index($i,",")-1)
		       ARGPOS = i
		    } else {
	           COMMENT[j] = ""  # newly inserted 
		       ARG[j]=substr($i,1,index($i,")")-1)
		       ARGPOS = i
            }
	        TYPE[j]=""
	        # build type specifier:
	        T = $(NEXT_TOKEN_INDEX)  # $(i-2) 
	        if (T ~ /\(/ && substr(T,0,index(T,"(")-1)==FNAME) {	    
	           # deal with special case that first type specifier 
		       # is directly adjacent to "name(":   
	           pos=index(T,"(")
	           TYPE[j]=substr(T,pos+1,length(T)-pos)
	           NEXT_TOKEN_INDEX++
	        } 
	        for (k=NEXT_TOKEN_INDEX; k<ARGPOS; k++) {
	           TYPE[j]=TYPE[j]" "$k 
	        }
            NEXT_TOKEN_INDEX = ARGPOS+1
		    if ($i==",") NEXT_TOKEN_INDEX++  # skip single comma
	        if (NEXT_TOKEN_INDEX>NF) NEXT_TOKEN_INDEX=1
	     	j++
         }
	  }
      # HERE WE DETECT END-condition for parsing arguments:
	  if (($i ~ /\)[^,\(]*$/ && substr($i,1,1)!="(") || $i == "*/)" || $i == "*/);" || VOID_ARGS) {
	     # now end of parameter list reached, i.e. definition
         # of a function prototype is completed. 
         # Two main cases:
         # i) the parsed prototype def is followed by another
         #    def that must be parsed. In this case, either
         #    $i ~ /.*;/ or $(i+1) == ";":
         if ($i ~ /.*;/ || (i<NF && $(i+1)==";") || (VOID_ARGS && !VOID_END)) {
            STATUS=EXPECT_FUNCTION_DEF
         }
         # ii) otherwise, this was the last prototype, and
         #    the remaining text will be comment and c-source:
         else {
            STATUS = PARSE_MAIN_COMMENT
            FIRST_PART_OF_MANPAGE = 0
         }
         # Anyway, before continuing, we still have to print
         # the parsed function prototype def:
	 if (PRINT_PROTOTYPE) {
            printf "\n" > FILE
            # print PROTOTYPE only once at the beginning:
            if (STATUS == PARSE_MAIN_COMMENT) {
                printf "%sPROTOTYPE%s\n",BEGIN_H2,END_H2 > FILE
            } else {
                printf "%sPROTOTYPES%s\n",BEGIN_H2,END_H2 > FILE  # another one follows
	    }
         } else print BEGIN_PAR > FILE  
         PRINT_PROTOTYPE="";
         PARLIST=""
         # make the parameter list for the prototype definition: 
	     if (VOID_ARGS) PARLIST="void"
         else for (k=0; k<j; k++) {
	        if (TYPE[k]=="" && ARG[k]=="") break; # done
            if (k==0) PARLIST=TYPE[0]" "ARG[0]
            else PARLIST=PARLIST", "TYPE[k]" "ARG[k]
         }
         # and print it:  HR@@@ one cr too much!
	     if (j>0 || VOID_ARGS) {
            if (LANGUAGE=="tex") {
               FTYPE   = DO_TEX_ESCAPES(FTYPE);
               FNAME   = DO_TEX_ESCAPES(FNAME);
               PARLIST = DO_TEX_ESCAPES(PARLIST);
            }
            print  BEGIN_TT FTYPE" "FNAME"("PARLIST")"END_TT BEGIN_PAR "\n" > FILE
            FTYPE = ""  
         }
         # add the newly found types/parameters/contents to the
         # argument description section:
         if (!VOID_ARGS) for (k=0; k<j; k++) {
            idx = ARG_INDEX[ARG[k]]
            # discard possible leading blank: 
            if (substr(ARG[k],1,1)==" ") ARG[k]=substr(ARG[k],2);
            if (substr(TYPE[k],1,1)==" ") TYPE[k]=substr(TYPE[k],2);
            if (idx != "" && \
                ARG_ENTRY[idx]==ARG[k] && TYPE_ENTRY[idx]==TYPE[k]) {
                  # if argument and type name are equal, add new comment
                  # to comment for this argument:  
                  COMMENT_ENTRY[idx] = COMMENT_ENTRY[idx]" "COMMENT[k]
            } else {
                # otherwise make new entry: 
                LAST_ENTRY++
                ARG_ENTRY[LAST_ENTRY] = ARG[k]
                TYPE_ENTRY[LAST_ENTRY] = TYPE[k]
                COMMENT_ENTRY[LAST_ENTRY]= COMMENT[k]
                ARG_INDEX[ARG[k]] = LAST_ENTRY
            }
         }
         j = 0   #  for parsing next prototype
         # clear entries collected for this prototype:
         for (k=0; k<LAST_ENTRY; k++) {
            COMMENT[k] = ""
            ARG[k] = ""
            TYPE[k] = ""
         }
         CONTINUE_COMMENT=""
         if (STATUS == PARSE_MAIN_COMMENT && !VOID_ARGS && PRINT_ARGS) {
            # print out collected argument descriptions: */
            printf "%sARGUMENTS%s\n",BEGIN_H2,END_H2  > FILE
            PrintArgList()
	     }  # if STATUS = PARSE_MAIN_COMMENT ..
	  }  # block dealing with end of parameter list ends here
      if (VOID_ARGS && VOID_END) { 
          STATUS = PARSE_MAIN_COMMENT; 
          next;
      }
      if (STATUS == EXPECT_FUNCTION_DEF) next;
    } # for
# -------------
# printf "%s", BEGIN_PAR > FILE
#   if (LANGUAGE != "nroff") printf "%s", BEGIN_PAR > FILE
    next;  # NEW
  } # block dealing with parsing FUNC_ARGS ends here
}

##########################################################################
#
#   End of processing function def 
#
##########################################################################

(LAST_ENTRY && RequestsArguments($0)) { 
   if (PARLIST != "void") {
      PrintArgList();
      printf "%s\n", BEGIN_PAR > FILE
   }
   next
}

/^\/[\*]+/,/[\*]+\//  {  # for all lines between /* .. */ inclusive: 

if (STATUS==PARSE_MAIN_COMMENT) {
 
   if (FINISH_ITEMS && !isManItem($0)) {
      printf "%s", FINISH_ITEMS > FILE
   }
   FINISH_ITEMS = "" 

   if (CVERBATIM ==0 && (PREVIOUS~/^[ ]*$/)) {
      CVERBATIM = 1    
   }

   if (CVERBATIM && ($0 !~ /^($| $|  $|   (.)*$)/) ) {
      if (CVERBATIM>1) print END_PRE > FILE
      CVERBATIM = 0;
   } 

   if (CVERBATIM && (substr($0,1,3) == "   ")) {
      if (CVERBATIM++ == 1) print BEGIN_PRE > FILE
      # in verbatim block, replace /@  and @/  by /*  and */:
      while (jj=index($0,"/@")) $0 = substr($0,1,jj-1)"/*"substr($0,jj+2);
      while (jj=index($0,"@/")) $0 = substr($0,1,jj-1)"*/"substr($0,jj+2);
      # for (i=1; i<=NF; i++) gsub(/[/]@/,"/*",$i); 
      # for (i=1; i<=NF; i++) gsub(/@\//,"*/",$i);  # hack to have comments
      printf "%s\n", $0 > FILE
      next
   } 
   
   if (!CVERBATIM) {
       if (AnnouncesPrototype1($NL)) {
           # New: Tokens C-SYNTAX or PROTOTYPE at end of 
           # man comment block allow to insert function parse
           STATUS              = EXPECT_FUNCTION_DEF
           ACCEPT_NEXT         = 0
           EXPECT_CONTINUATION = 0
           PRINT_ARGS          = 0
           PRINT_PROTOTYPE     = 0
           ClearArgList();
           next
      }
      iLen = length($(NF))
      if (LANGUAGE=="tex") {
        # DO_TEX_ESCAPES:  don't use function here, must be done "in place"!!:
        TEXMATH=0;
        if (substr($1,1,2)=="$$" && NF>1 && substr($(NF),iLen-1,2)=="$$") {
           # display math detected:
           # unsubstitute inner dollar signs:
           iLen = length($0)
           for (jj=3; jj<=iLen-2; jj++) {
                 if (substr($0,jj,1)=="$") {
                    $0 = substr($0,1,jj-1)" "substr($0,jj+1)
                 }
           }
           # replace conditional braces by normal braces:
           while (jj=index($0,"`{")) $0=substr($0,1,jj-1)substr($0,jj+1)
           while (jj=index($0,"}'")) $0=substr($0,1,jj)substr($0,jj+2)
           # resplit current line:
           NF=split($0,Fields);
           for (i=1; i<=NF; i++) $i = Fields[i]
           if (NF>2 && $(NF-1) ~ /\([0-9]+\)/) {
              # last field is equation nr: insert a \hfil:
              $(NF-1)="\\hfil"$(NF-1)
           }
        } else for (i=1; i<=NF; i++) {
           iMathSymbol = index($i,"$");
           # look for a single $ at the start or end of $i, but
           # not prefixed with a backslash!
           if (iMathSymbol==1) {
              TEXMATH=1-TEXMATH
              iMathSymbol = index(substr($i,2),"$");
              # token may contain math symbol at both beginning
              # and end! 
              if (iMathSymbol) iMathSymbol++; 
              # +1, to have position w.r.t $i
           }
           if (TEXMATH) {
              # unsubstitute inner dollar signs:
              iLen = length($i)
              if (iMathSymbol==iLen-1 && iMathSymbol>1) iLen--;
              # exclude prev to last $ from substitution (eg, in $..$. and $..$,)!
              for (jj=2; jj<iLen; jj++) {
                 if (substr($i,jj,1)=="$") {
                    $i = substr($i,1,jj-1)" "substr($i,jj+1)
                    # don't change length! Would spoil comparison below!
                 }
              }
              if (iMathSymbol>=length($i)-1 && (iMathSymbol>1) && substr($i,iMathSymbol-1,1)!="\\") TEXMATH=0
              # replace conditional braces by normal braces:
              while (jj=index($0,"`{")) $0=substr($0,1,jj-1)substr($0,jj+1)
              while (jj=index($0,"}'")) $0=substr($0,1,jj)substr($0,jj+2)
              continue 
           }
           # FURTHER TEX_ESCAPES I:
           # if not in math mode, protect underscores and ^-s:
           # deal with underscore _ :
           gsub(/_/,"\\@#!~",$i); 
           gsub(/\\@#!~/,"\\_",$i); 
           # deal with quoted underscore \_ :
           gsub(/\\\\_/,"\\_",$i);
           gsub(/\^/,"\\@#!~",$i); 
           gsub(/\\@#!~/,"\\^",$i); 
           gsub(/\\\\^/,"\\^{}",$i); # ^ requires \^{} in TeX!!
           # deal with tilde ~ :
           gsub(/\~/,"\\@#!~",$i); 
           gsub(/\\@#!~/,"\\~~",$i); 
           if (jj=index($i,"&")) {   # gsub doesnt work for ampersand!
              Length = length($i)
              while (jj<=Length) {
                 if (substr($i,jj,1)=="&") {
                    # protect ampersand by inserting backslash:
                    $i = substr($i,1,jj-1)"\\"substr($i,jj)
                    jj = jj+2
                    Length++
                 } else jj++
              }
           }
        }
        # FURTHER TEX_ESCAPES II:
        # deal with % :
        for (i=1; i<=NF; i++) gsub(/%/,"\\@#!~",$i); 
        for (i=1; i<=NF; i++) gsub(/\\@#!~/,"\\%",$i); 
        for (i=1; i<=NF; i++) gsub(/\\\\%/,"\\%",$i);
        # deal with # :
        for (i=1; i<=NF; i++) gsub(/#/,"\\@;!~",$i); 
        for (i=1; i<=NF; i++) gsub(/\\@;!~/,"\\#",$i); 
        for (i=1; i<=NF; i++) gsub(/\\\\#/,"\\#",$i);
        # deal with \{ :
        for (i=1; i<=NF; i++) gsub(/\\{/,"\\@#!~",$i); 
        for (i=1; i<=NF; i++) gsub(/\\@#!~/,"$\\{$",$i); 
        # deal with \} :
        for (i=1; i<=NF; i++) gsub(/\\}/,"\\@#!~",$i); 
        for (i=1; i<=NF; i++) gsub(/\\@#!~/,"$\\}$",$i); 
        # deal with \$ :
        for (i=1; i<=NF; i++) gsub(/\\\$/,"\\@#!~",$i); 
        for (i=1; i<=NF; i++) gsub(/\\@#!~/,"\\$",$i); 
      } else {  # replace some tex-escapes:

         #  asciified display math: just insert par break:
         if (substr($1,1,2)=="$$" && NF>1 && substr($(NF),iLen-1,2)=="$$") {
            printf "%s\n", BEGIN_PAR BEGIN_CITE > FILE
            $0   = substr($0,3,length($0)-4) END_CITE BEGIN_PAR 
            # resplit current line:
            NF=split($0,Fields);
            for (i=1; i<=NF; i++) $i = Fields[i]
         }

         # remove some backslash quotes:
         # Note: for strange reasons, gsub will not work here!
         while (jj=index($0,"\\_")) $0=substr($0,1,jj-1)substr($0,jj+1)
         while (jj=index($0,"\\%")) $0=substr($0,1,jj-1)substr($0,jj+1)
         while (jj=index($0,"\\#")) $0=substr($0,1,jj-1)substr($0,jj+1)
         # resplit current line:
         NF=split($0,Fields);
         for (i=1; i<=NF; i++) $i = Fields[i]
         # the following three lines remove single {'s, but keep 
         # {'s prefixed by a backslash:
         for (i=1; i<=NF; i++) gsub(/\\{/,"\\@#!~",$i);
         for (i=1; i<=NF; i++) gsub(/{/,"",$i);
         for (i=1; i<=NF; i++) gsub(/\\@#!~/,"{",$i);
         # the following three lines remove single }'s, but keep 
         # }'s prefixed by a backslash:
         for (i=1; i<=NF; i++) gsub(/\\}/,"\\@#!~",$i);
         for (i=1; i<=NF; i++) gsub(/}/,"",$i);
         for (i=1; i<=NF; i++) gsub(/\\@#!~/,"}",$i);

         # the following three lines remove single $'s, but keep 
         # $'s prefixed by a backslash:
         for (i=1; i<=NF; i++) gsub(/\\\$/,"\\@#!~",$i);
         # note: $ is special for regexpr & needs prefix!
         for (i=1; i<=NF; i++) gsub(/\$/,"",$i);
         for (i=1; i<=NF; i++) gsub(/\\@#!~/,"$",$i);

      }
   }
   # print "!"$0

   if ($0 ~ /^@ARGUMENTS[: ]*$/) {
      if (LAST_ENTRY > 0 || PARLIST=="void") {
         if (PARLIST != "void") {
            PrintArgList()
            printf "%s\n", BEGIN_PAR > FILE
         }
         next
      } else {
         # Alternative interpretation, if no function args were parsed:
         # a line starting with "@ARGUMENTS .. indicates the beginning
         # of an ARGUMENT-specification block, i.e. a text block that 
         # that specifies command line parameters in
         # a format that is readable for the mkopt-routine.
         # Reformat this info suitably for the man-page:
         PARSE_ARGUMENTS = 1 
         $1 = substr($0,2)    # removes initial @-char
         printf "%s%s%s\n",BEGIN_H2,$0,END_H2 > FILE
         printf BEGIN_LIST  > FILE  # start description list
         PREVIOUS = $0
         next
      }
   }
   if (PARSE_ARGUMENTS == 1) {
      if ($0 ~ /^[A-Z]/) {
         # any line that starts with a capital letter ends
	     # the ARGUMENT-specification block.
         PARSE_ARGUMENTS=0
         if (FILENAME ~ /.+\.c/)  {
           # for c-source files only:
	       printf "%s-@ %sspecify init file\n",BEGIN_TAG,BEGIN_DESCRIPTION > FILE
	     }
         print END_LIST > FILE  # end description list
      } else { 
         # process a line in the ARGUMENT-specification block
         for (i=1; i<=NF; i++) {
            if ($i == "#") break;  # find #-sign that separates specification
	                        # from comment text
         }
         if (substr($2,1,4)==LT_CHAR) {
            NOSPACE = FOOTNOTE
	        NOSPACE_OCCURRED = 1
         } else NOSPACE = ""
         for (j=2; j<=i; j++) $j = ""
         if (k=index($NF,"\\")) $j=substr($j,1,k-1)"<BR>"
	     for (j=i+1; j<=NF; j++)  # make dot set em-font
            if (substr($j,1,1)==".") {
               $j =  EMPHASIZE($j)  # BEGIN_CITE substr($j,2) END_CITE 
            }
         FLAG = $1    # name of the option 
         $1 = ""      
         if (i<=NF) {
            if (i>1) printf "%s%s%s%s%s%s\n",BEGIN_TAG,BEGIN_TT CITE_SPACE,FLAG,END_TT,BEGIN_DESCRIPTION,$0 > FILE
            else printf "%s\n", $0 > FILE
         }
         if (PARSE_ARGUMENTS) {
            PREVIOUS = $0
            next
         }
      }
   }
   if (SECTIONTRUE || $0 ~ /^([A-Z0-9]|\\")[-A-Z a-z_0-9()\.\\"]*\:[ ]*$/) {
      # any line of form ^TITLE: becomes a manSection:
      if ($NF==":") $NF=""  # isolated ":" is suppressed
      printf "%s%s%s\n",BEGIN_H2,$0,END_H2 > FILE  
      PREVIOUS = $0
      SECTIONTRUE=0  # reset after printout
      next
   } 

   if (SUBSECTIONTRUE || $0 ~ /^([-A-Z 0-9()]|\\")(.)*\::[ ]*$/) {
      # any line of form ^TITLE:: becomes a manSubSection:
      if ($NF=="::") $NF=""  # isolated "::" are suppressed
      if (index($NF,"::")) $NF=substr($NF,1,length($NF)-1) # remove last :
      for (i=1; i<=NF; i++) {
          # make dot set \tt-font
          if (substr($i,1,1)==".") $i =  EMPHASIZE($i) # BEGIN_CITE substr($i,2) END_CITE 
      }
      printf "%s%s%s\n",BEGIN_H3,$0,END_H3 > FILE  
      if (LANGUAGE=="nroff") {
      }
      PREVIOUS = $0
      SUBSECTIONTRUE=0  # reset after printout
      next
   }

   ##############   detect manItems:    # { to balance brace below 

   if ((LANGUAGE == "html" && $0 ~ /^(<:)*\-[ ]*(.|\n)*\:(<[/]CITE>|$| [ ]*)/) ||
       (LANGUAGE == "tex"  && $0 ~ /^(<:)*\-[ ]*(.|\n)*\:(}|$| [ ]*)/)         ||
                             ($0 ~ /^(<:)*\-[ ]*(.|\n)*\:(\\fR|$| [ ]*)/)     ) { 
      ### inserted <[/]CITE> to take care of already tt-ed tokens!
      if (!ITEMTRUE) {
         printf "%s",BEGIN_LIST > FILE  # start description list
         ITEMTRUE=1
      } 
      # any line of form ^-TITLE: becomes a manItem:
      ITEM = ""
      $1 = substr($1,2)  # discard initial minus sign
      for (i=1; i<=NF; i++) {
          # make dot set \tt-font
          if (substr($i,1,1)==".") $i =  EMPHASIZE($i) # BEGIN_CITE substr($i,2) END_CITE 
          if ($i == ":") {
             break
          }
          ITEM = ITEM" "$i
#         if ($i ~ /\:/ ) break
          if (substr($i,length($i),1)==":") break
      }
      REST = ""
      for (j=i+1; j<=NF; j++) {
         # make dot set \tt-font
         # if (substr($j,1,1)==".") $j =  EMPHASIZE($j)  # BEGIN_CITE substr($j,2) END_CITE 
         REST = REST" "$j 
      }
      # discard initial minus sign in ITEM:
      if (LANGUAGE == "nroff") {  
         if (REST) { 
            printf "%s%s%s",BEGIN_TAG,substr(ITEM,2),BEGIN_DESCRIPTION > FILE
            printf "%s\n", substr(REST,2) > FILE
         } else {
            printf "%s%s\n",BEGIN_TAG,substr(ITEM,2) > FILE
         }
      } else {
         printf "%s%s%s",BEGIN_TAG,substr(ITEM,2),BEGIN_DESCRIPTION > FILE
         printf "%s\n", substr(REST,2) > FILE
      }
      PREVIOUS = $0
      next
   } else {
      if (ITEMTRUE && ($0 ~ /^ *$/)) {
#        print END_LIST > FILE   # print introduces additional newline!!
         printf "%s", END_LIST > FILE
         if (LANGUAGE == "nroff") FINISH_ITEMS = ".PP\n"
         else FINISH_ITEMS = "\n"
         ITEMTRUE=0  # blank line resets item mode
      }
   }
   if ($0 ~ /^ *$/) {
      /* blank line */
      if (ITEMTRUE) { 
#        print END_LIST > FILE
         printf "%s" END_LIST > FILE
         ITEMTRUE=0  # blank line resets item mode
      } else if (!CVERBATIM) {
#        print BEGIN_PAR > FILE
         if (LANGUAGE == "nroff" && FINISH_ITEMS) {
            PREVIOUS = $0
            next
         }
      }
   }
      
   if ($1 !~ /\/[\*]+/ && $NF !~ /[\*]+\//) {
      if (!CVERBATIM) for (i=1; i<=NF; i++) 
         if (substr($i,1,1)==".") $i =  EMPHASIZE($i) # BEGIN_CITE substr($i,2) END_CITE 
      printf "%s\n", substr($0,match($0,/[^ ]/)) > FILE   # omit leading spaces
      PREVIOUS = $0
   }
   if (IsManEnd($0)) {
      # @MANEND forces end of current man page:
      STATUS                = 0 
      ACCEPT_NEXT           = 0
      EXPECT_CONTINUATION   = 0
      FIRST_PART_OF_MANPAGE = 0
      next
   }
   if ($NF ~ /[.]*[\*]+\//) {
        # end of a comment:
        if (FIRST_PART_OF_MANPAGE) {
           FIRST_PART_OF_MANPAGE = 0
           STATUS = EXPECT_FUNCTION_DEF
           ACCEPT_NEXT           = 1
           EXPECT_CONTINUATION   = 0
        } else {
           STATUS                = 0 
           ACCEPT_NEXT           = 0
           EXPECT_CONTINUATION   = 1
        }
        next
   }
} 
}

END {
   if (MAN_TITLE != "") {
      if (index(PATTERN,FIRST_MANTITLE" ") + DO_ALL) {
         FinishPreviousManpage(TEXT_PART)
      }
   } 
   if(dep_file_list) { print "MANFILES +=" dep_file_list ; }
} 	
