#ifndef __NEO_TYPES_T__
#   define __NEO_TYPES_T__

#include "Nst_aux.h"
#include "Neo_dialog.h"

#define MAXLABELEN 25

#define NEO_CMD_END       0
#define NEO_CMD_CONTINUE -1
#define NEO_CMD_BREAK    -2

#define InitCommand       (1L<<1)
#define IconSelect        (1L<<2)
#define IconOptionSelect  (1L<<3)
#define IconInputSelect   (1L<<4)
#define IconOutputSelect  (1L<<5)
#define IconRemove        (1L<<6)
#define WireSelect        (1L<<7)
#define AnnotationSelect  (1L<<8)
#define IconScopeSelect   (1L<<9)
#define CommandSelect     (1L<<10)
#define KeyEvent          (1L<<12)
#define UnknownEvent      (1L<<13)
#define Button1Pressed    (1L<<14)
#define Button2Pressed    (1L<<15)
#define Button3Pressed    (1L<<16)
#define Button1Released   (1L<<17)
#define Button2Released   (1L<<18)
#define Button3Released   (1L<<19)
#define PointerMoved      (1L<<20)
#define RemoveCommand     (1L<<21)
#define EnterWindow       (1L<<22)
#define LeaveWindow       (1L<<23)
#define KeyPressed        (1L<<24)
#define KeyReleased       (1L<<25)

typedef  void* cmdptr;
typedef  void neo_status_t;

typedef struct neo_icondef_ neo_icondef_t;

typedef struct neo_fmt_entry_{
   char *pcName;  char *pcFmt;
   struct neo_fmt_entry_ *next;
} neo_fmt_entry_t;

/* auxiliary macros to access private data of IconDef_t instance: */
#define FLAGS_OF(C)  ((neo_icondef_private_t*)((C)->pPrivate))->iFlags
#define NUMREFS_OF(C)  ((neo_icondef_private_t*)((C)->pPrivate))->iNumRefs
#define CREATED_OF(C)  ((neo_icondef_private_t*)((C)->pPrivate))->iCreated
#define FMTLIST_OF(C)  ((neo_icondef_private_t*)((C)->pPrivate))->pFmtList

typedef struct neo_icondef_private_{
   unsigned int iFlags; /* flags */
   int      iCreated;   /* nr of icons of this type created so far */
   int      iNumRefs;   /* nr of references to this icon type      */
   dialog_var_t *pCreateVar, *pConfigVar;
   neo_fmt_entry_t *pFmtList;
} neo_icondef_private_t;

typedef struct neo_icon_{
      struct neo_icon_ *next,*prev,*host;
      struct neo_icon_ *first_subunit,*last_subunit;
      struct neo_wire_ *first_wire,*last_wire;
      char *pcOrig;          /* id of associated NST unit */ 
      char *pcClassName;     /* class name as string */ 
      int iXll, iYll;        /* icon position */
      int iAddToDy;          /* if icon has not standard height */
      unsigned long iStatus; /* flags */
      int iUpdates;          /* update count */
      int iNum;              /* nr parameter */
      int iNumInp, iNumOut;  /* nr of input and output ports */
      int  *piInpY,*piOutY;  /* vertical pixel positions of ports */
      int  *piInpType,
           *piOutType;       /* types of ports */
      int  *piInpNums,
           *piOutNums;       /* nrs of pins in ports */
      char *pcInp,*pcOut;    /* string description of input/output ports */
      int  iCtlField;        /* pos of ctl field, if any, or -1 */
      char *pcName;          /* icon name */
      char *pcOldName;       /* old icon name */
      char *pcLabel;         /* icon label */
      char *pcTitle;         /* icon title */
      char *pcOption;        /* option fields on icon */
      char *pcOptionColor;   /* colors for option fields */
      char *pcManPage;       /* colors for option fields */
      neo_dialog_t *pDefWdw; /* definition window */     
      neo_dialog_t *pUserDefWdw; /* user-defined definition window */  
      float *pfUserDefVal;   /* data in UserDefWdw: float values */   
      char **ppcUserDefVal;  /* data in UserDefWdw: text strings */   
      neo_dialog_t *pCtlWdw; /* control window */  
      float *pfCtlVal;       /* data in CtlWdw: float values */   
      char **ppcCtlVal;      /* data in CtlWdw: text strings */   
      neo_dialog_t *pMetaWdw; /* meta window */
      char *pcUserFmt;       /* user window format */
      char *pcDummySave1,    /* if icon is a dummy icon: saved file */
           *pcDummySave2;    /* section data of orig icon */
      neo_icondef_t  *psFun; /* associated NST constructor fn */    
      void    *psScope;
      void    *pAnnotation;  /* annotation ptr */
} neo_icon_t, *iconptr;

typedef struct neo_wire_def_{
   int iRefs;    /* nr of references to this wire definition */
   int iDimOut;  /* dimension of destination field at which this wire ends */
   int *piSrcOf; /* array of DimOut pin indexes.
                SrcOf[i] = if >=0:
                              position of src pin in src field feeding to
                              destination pin at position i.
                           if -1:
                              dest pin at pos i has not yet any writer.
             */
   struct neo_wire_ **ppWireThatFeeds;
      /* if dest pin i has a writer pin, the connection need
         not be part of THIS wire. The wire to which a connection
         to dest pin i belongs is always given by WireThatFeeds[i].
      */
} neo_wire_def_t;

typedef struct neo_wire_{
   struct neo_wire_ *next,*prev;
   iconptr pSrcIcon, pDestIcon, pHost;
   char *pcWire;  /* adr of entry in server */
   int iColor,iDestIsOut,iSrcIsOut,iDestField,iSrcField;
   /* wire geometry: */
   float   fX,fY,fLen;
   /* defined connections: */
   neo_wire_def_t *pDef;
} neo_wire_t, *wireptr;

typedef struct neo_event_{
      int iType;               /* event type, one of the above 11 */
      int iPos;                /* if: ICONSELECT:       queue pos of icon
                                      ICONREMOVE:       queue pos of icon
                                      ICONOPTIONSELECT: pos of option
                                      ICONINPUTSELECT:  pos of inp field
                                      ICONOUTPUTSELECT: pos of out field
                                      ICONOUTPUTSELECT: pos of out field
                                      ANNOTATIONSELECT: flag
                                      COMMANDSELECT:    button pos
                               */
      int iX, iY;              /* abs mouse coordinates in window
                                  (0,0) = lower left              */
      int iXPrev, iYPrev;      /* mouse coord at previous drag event */
      struct {
         iconptr icon;         /* if an icon, icon option or an input
                                  or output field on an icon was selected */
         void *wire;           /* if a wire was selected */
         void *annotation;     /* if an annotation was selected */
         cmdptr  command;      /* if a command was selected */
         int     iconoption;   /* if an icon option was selected */
         int     scope;        /* +-1 if a scope rectangle was selected.
                                  in this case, icon is set to the corresp.
                                  icon.  */
         int     port;         /* != 0 if a port rectangle was selected.
                                  abs(port)-1 = field nr, inp/out for -/+ */
      } selected;
      char *pcButtonLabel;
      void *pEvent;                /* ptr to provide access to raw event struct */
      int iButton1,iButton2,iButton3;
      unsigned long iKeySym;       /* key symbol of key that changed state */
      /* the remaining might be in a union: */
      int iXRel, iYRel;            /* mouse coord within icon area */
} neo_event_t;

/* the following contain everything exported from Neo_tk.c: */

extern unitptr neo_host(void);
extern void neo_main_loop(void);
extern void *neo_add_event_cmd(int(*)(neo_event_t*,void*),neo_status_t*,int,char*);
extern void *neo_add_button_cmd(int(*)(neo_event_t*,void*),neo_status_t*,char*,char*);
extern char *neo_get_button_label_of(cmdptr);
extern int  neo_get_button_pos_of(cmdptr);
extern void neo_set_button_pos_of(cmdptr,int);
extern void neo_delete_raw_window(void *pWindow);

#endif /* __NEO_TYPES_H__ */
