/******************************************************************************
 *
 * neo4.h  -  headers for neo4.c
 *
 * Copyright Helge Ritter, 1994
 *
 * $Source: /vol/ni/src/cvsroot/nst5/neosrc/neo4.h,v $
 * $Log: neo4.h,v $
 * Revision 1.15  2005/09/02 13:44:08  rhaschke
 * merge with Helge's NST7 version: simplified event loop, smaller neo-files
 * major cleanup of compiler switches, declarations, etc.
 *
 * Revision 1.14  2002/03/04 15:43:56  rhaschke
 * removed makeMetaWdw from export list (it is static in neo8!)
 *
 * Revision 1.13  2000/02/07 17:20:36  helge
 * - some adjustments to support new exec_container_opnds() call of prog_unit
 *   with interactively adjustable scope rectangle around container
 * - bug correction (inconsistent scope updates when several virtual_for_op's
 *   were in the same container)
 * - ManPage window no longer blocking
 *
 * Revision 1.12  2000/02/01 16:04:46  helge
 * added %PRESERVE option for Neo's remake_unit (allows to exclude subicons
 * from remake operation)
 *
 * Revision 1.11  2000/01/10 18:20:28  helge
 * minor changes, removed warnings
 *
 * Revision 1.10  1999/03/09 14:00:41  helge
 * redefinition of NeoReadHeaderInfo() parameter list
 *
 * Revision 1.9  1999/02/23 10:46:19  helge
 * remaining changes for Neo cooperation with array_of unit
 *
 * Revision 1.8  1999/02/14 17:49:47  helge
 *  - neo_main.c icons can now become activated (executed) when the
 *    cursor moves into a window that has been associated with the
 *    icon before (specification occurs in Define dialog).
 *  - neo.h neo1.c neo2.c neo3.c neo4.c neo5.c neo8.c neo9.c neo10.c
 *    neo_main.c : changes to allow loading of icon units from loadable
 *    folders "by reference": such icons cannot be changed internally
 *    and they will always stay in sync with their definition in
 *    the folder.
 *  - neo_main: changed color initialization to newer rgb:rr/gg/bb
 *              format to avoid problems with the initial '#'-symbol (which
 *              unfortunately is already reserved as a comment symbol).
 *  - optimization of instantiation of loadable icons from folders:
 *    so far, a loadable icon was made with its default parameters,
 *    and then remade when changes were requested. Now the query for
 *    changes is made earlier (in NeoReadHeaderIcon()) so that changes
 *    can already become effective during the first instantiating
 *    process and no further remake cycle is necessary.
 * - minor cleanup: EditWire() is now NeoEditWire() (affects
 *   neo?.c, neo3.h, neo_stdr.c)
 *
 * Revision 1.7  1999/02/04 12:37:35  helge
 * Minor changes for improved (un-)drawing of cascaded pop-up menues
 *
 * Revision 1.6  1997/12/21 15:51:09  helge
 * minor adjustments
 *
 * Revision 1.5  1997/12/21 00:07:37  helge
 * Mainly changes to support major new feature of ``class containers'':
 * C++ classes can now be wrapped as NST units such
 * that a class instance appears as a ``class container'' with subunits
 * representing the methods.
 * Necessary wrapper code is generated with new tool mku++.
 * Similar procedure for C ``classes'' supported by new tool mku+
 *
 * Revision 1.4  1997/10/13 16:42:28  helge
 * - extensive clean-up in src-generator routines (mainly neo6.c)
 * - some bugs in neo remake_unit removed (neo11.c)
 * - better src generation, better diagnostics (neo1,2,main)
 * - some wrappers corrected   (mainly SRCLINEs)
 * - unit funct_gen now obsolete and hidden (but kept); use
 *   function_gen for new work (differs only in wrapper)
 *
 * Revision 1.3  1997/04/02 12:46:57  helge
 * several minor corrections, among them: better handling of operator scopes,
 * ~-expansion in .neofolders-file, unloading of dynamic folders now works,
 * save gz file now safeguarded against specification of non-existent dir.
 *
 *****************************************************************************/
#ifndef __NEO4_H
#define __NEO4_H

extern int giShiftOfCopy;

extern int BufferVarsOf(unitptr uIcon, unitptr *puBuffer);
extern int UnbufferVarsOf(unitptr uIcon, unitptr *puBuffer);
extern int MkUserWdwEntry(unitptr uWdw, int iNum, char *pcEntry);

extern int  NeoDefineCommand2WasCalled(void);
extern void NeoDisableDefineFunction(void);
extern void NeoEnableDefineFunction1(unitptr uIcon);
extern void NeoEnableDefineFunction2(unitptr uIcon);

extern char* NeoDefaultCommands(void);
extern void NeoExecuteCommand(char *pcCmd, unitptr uIcon);
extern void NeoExpandByReferenceDefinition(unitptr uIcon, int iRecursively);
extern char* NeoMakeCommandMenu(unitptr uIcon);
extern int   NeoTestDefaultCommands(char *pcStr);

extern unitptr derive_icon_from(unitptr uIcon, unitptr uHost);

#endif
