/******************************************************************************
 *
 * neo9.h  -  headers for neo9.c
 *
 * Copyright Helge Ritter, 1996
 *
 * $Source: /vol/ni/src/cvsroot/nst5/neosrc/neo9.h,v $
 * $Log: neo9.h,v $
 * Revision 1.7  2005/09/02 13:44:10  rhaschke
 * merge with Helge's NST7 version: simplified event loop, smaller neo-files
 * major cleanup of compiler switches, declarations, etc.
 *
 * Revision 1.5  2000/01/10 18:20:29  helge
 * minor changes, removed warnings
 *
 * Revision 1.4  1997/12/21 00:07:38  helge
 * Mainly changes to support major new feature of ``class containers'':
 * C++ classes can now be wrapped as NST units such
 * that a class instance appears as a ``class container'' with subunits
 * representing the methods.
 * Necessary wrapper code is generated with new tool mku++.
 * Similar procedure for C ``classes'' supported by new tool mku+
 *
 * Revision 1.3  1997/04/02 12:47:02  helge
 * several minor corrections, among them: better handling of operator scopes,
 * ~-expansion in .neofolders-file, unloading of dynamic folders now works,
 * save gz file now safeguarded against specification of non-existent dir.
 *
 * Revision 1.2  1997/01/14  14:35:23  helge
 * changes 135-139 bugs 131,132, numerous further small changes to
 * take care of (almost) all warnings under -Wall -O2
 * !!! Some Neo-structs have changed, trust the Makefile or  !!!
 * !!! recompile EVERYTHING !!!
 *
 *****************************************************************************/
#ifndef __NEO9_H
#define __NEO9_H

extern void NeoAddAnnotation(int iMx, int iMy, unitptr uIconBox);
extern void NeoAddAreaTagsForAnnotation(
   FILE *fp, int iWindowHeight,  char *pcFileName,   unitptr u );
extern unitptr NeoAnnotationUnitOf(unitptr uIcon);
extern unitptr NeoCopyOfAnnotationUnit(unitptr uTemplate, unitptr uHost);
extern int NeoCopySelectedAnnotations(unitptr uFromHost, unitptr uToHost);
extern int NeoDeselectAnnotations(unitptr uHost, int iDraw);
extern void NeoDoAnnotationCommands(void);
extern int NeoGetNumAnnotationsInQueue(unitptr uIconBox);
extern void NeoHTMLDocAnnotationUnit(FILE *fp, char *pcColor, unitptr u);
extern char* NeoIconManpageDefinesIdent(unitptr uIcon);
extern int NeoIsAnnotationUnit(unitptr u);
extern void NeoMoveAllAnnotations(unitptr uSrcIcon, unitptr uDestIcon);
extern int NeoMoveAnnotations(unitptr uFromHost, unitptr uToHost);
extern void NeoPrintAnnotations(
FILE *fp,  char* pcIndent, unitptr uAnnotation,  unitptr uIcon );
extern void NeoPrintManText(FILE *fp,int iBody,unitptr uIcon);
extern int NeoSelectAnnotationsInRegion(
int iXll, int iYll, int iXur, int iYur, unitptr uAnnotation);
extern char* NeoTitleOfAnnotationForField(int iField, int iIsOut, unitptr uHost);
extern int NeoTranslateSelectedAnnotations(
int iDx, int iDy, int iXlimit, int iYlimit, unitptr uAnnotation);
extern int PrependManpage(FILE *fp, unitptr uIcon);
extern void ReplaceVarsByValuesForAnnotation(char *pcWhich, unitptr u);
extern char* aux_read_text_lines(FILE *fp, char *pcFirstLine);
extern void aux_write_text_lines(FILE *fp, int iIndent, char *pcText);
extern int load_annotation_unit(FILE *fp, unitptr uHost);

/* The following is used to update annotations of container fields */
extern void NeoStoreAnnotationsOfFields (
   unitptr uHost, int **ppiInpFields, int **ppiOutFields);
extern void NeoChangeAnnotationsOfFields (
   unitptr uHost, int *aiFieldArray, int iField, int mode);
extern void NeoUpdateAnnotationsOfFields (
   unitptr uHost, int *piInpFields, int *piOutFields);

#define DELETE_FIELD    0
#define INSERT_FIELD    1
#define MOVE_UP_FIELD   2
#define MOVE_DOWN_FIELD 3

#endif /* __NEO9_H */
