#ifndef __NEO_TYPES_H
#define __NEO_TYPES_H

#include "neo.h"
#include "glX11.h"
#include "nst_X11.h"

#define NEOMAXNAMELEN 256  /* max len for file names, paths etc. */

/* define some defaults: */

#define MANDIR_NROFF      "man/cat3"
#define MANSUFFIX_NROFF   ".3"
#define MANDIR_HTML       "man/html3"
#define MANSUFFIX_HTML    ".html"
#define EXAMPLE_DIR       "man/examples"
#define ICON_BY_REFERENCE "--BY_REFERENCE--" 

/* some shorthands: */

#define BUTTON(u,t,i) (Y(u,0,i)>0)  /* t is just for comment */
#define VALUE(u,t,i) (Y(u,0,i))     /* t is just for comment */

/* some cursor shapes: */
#define NORMALARROW  1
#define WATCHSYMBOL  2
#define XCURSOR      5
#define HOLLOWCIRCLE 4

#define NEOMAXCIRCUITS  5  /* max nr of simultaneous circuits */
#define ICONWINDOWS     5  /* nr of windows between which icons may be moved */
#define SOURCEBUFLEN 2000  /* no single piece of source code may be longer
                              than this [no checking!!] */

/****** still poor: probably only correct for grayscale LCD monitor! ******/

#ifdef GRAYSCALE  
#define FEEDBACK_COLOR      0    /* color for feedback objects */
#define COLOR_OF_SELECTED_WIRE 7 /* color indicating selected wire */
#define ICON_COLOR          1    /* color of an icon: same as wdw */
#define ICON_SELECT_COLOR   2    /* */
#define LINE_COLOR          0    /* color for drawing lines on an icon */
#define PACKED_FIELD_COLOR  8  
#define NORMAL_FIELD_COLOR  7    /* same as Text: white */ 
#define TEXT_FIELD_COLOR    7 
#define BACKGROUND_COLOR   BUTTCOL2 /* defines a general background color */
#define BACKGROUND_COLOR1  SLICOL1   /* background color for unmodifyable circuits */
#else    
#define FEEDBACK_COLOR         0        /* color for feedback objects */
#define COLOR_OF_SELECTED_WIRE WHITE    /* color indicating selected wire */
#define ICON_COLOR             BUTTCOL1 /* color of an icon: same as wdw */
#define ICON_SELECT_COLOR      SLICOL2  /* */
#define LINE_COLOR             BLACK    /* color for drawing lines on an icon */
#define PACKED_FIELD_COLOR     BLACK  
#define NORMAL_FIELD_COLOR     WHITE    /* same as Text: white */ 
#define TEXT_FIELD_COLOR       WHITE 
#define BACKGROUND_COLOR       BUTTCOL2  /* defines a general background color */
#define BACKGROUND_COLOR1      SLICOL1  
                                     /* background color for unmodifyable circuits */
#endif

#define FLOAT_IDENT "fArg"           /* float var identifier f. src-gen. */
#define TEXT_IDENT  "tArg"           /* text var identifier f. src-gen.*/
#define INT_IDENT   "iArg"           /* int var identifier f. src-gen.*/

#define P_has_manpage      1U
#define P_has_example      2U
#define P_has_parameters   4U
#define P_hide_icon        8U
#define P_is_dynamic      16U
#define P_allows_modify   32U
#define P_is_container    64U
#define P_is_loadable    128U
#define P_forbid_ref     256U

/* flags for individual icons: */
#define I_is_selected      1U
#define I_is_by_ref        2U
#define I_is_highlighted   4U
#define I_is_picked        8U
#define I_skip_data        8UL

#define ICONDX 80
#define ICONDY 80

typedef struct neo_rect_{
  int iXMin,iXMax,iYMin,iYMax;
} NeoRect_t;

typedef struct def_NstIconWdwDef_t{
   int nr; 
   char *title, *options; 
   int level, wdw,active,resize,col,xll,yll,w,h;
   unitptr iconbox;
} NstIconWdw_t;

typedef struct def_NstIconData_t{
   int xll,yll,w,h,col,but;
   char *icon;
} NstIconData_t;

typedef struct NstFmtEntry_{
   char *pcName;
   char *pcFmt;
   char *pcParFmt;
   int (*pConfigFunction)(int,int,NstIconDef_t*,unitptr); 
   struct NstFmtEntry_ *next;
} NstFmtEntry_t;

/* auxiliary macros to access private data of IconDef_t instance: */
#define FLAGS_OF(C)  ((NstIconDefPrivate_t*)((C)->pPrivate))->iFlags
#define NUMREFS_OF(C)  ((NstIconDefPrivate_t*)((C)->pPrivate))->iNumRefs
#define CREATED_OF(C)  ((NstIconDefPrivate_t*)((C)->pPrivate))->iCreated
#define FMTLIST_OF(C)  ((NstIconDefPrivate_t*)((C)->pPrivate))->pFmtList
#define FOLDER_OF(C)  ((NstIconDefPrivate_t*)((C)->pPrivate))->iFolder

typedef struct NstIconDefPrivate_{
   unsigned int iFlags; /* flags */
   int      iCreated;   /* nr of icons of this type created so far */
   int      iNumRefs;   /* nr of references to this icon type      */
   int      iFolder;    /* index of folder */
   int      aiVacancySize[NEOMAXCIRCUITS];
   unsigned char *apcVacancyBits[NEOMAXCIRCUITS];
   unitptr  uLastCreateWdw, uLastConfigWdw;
   NstFmtEntry_t *pFmtList;
} NstIconDefPrivate_t;

typedef struct IconVar_{
   unitptr uOrig;          /* ptr to NST-unit that is wrapped by this icon  */
   unitptr uDefWdw;        /* ptr to dialog window for defining this unit   */
   unitptr uUserDefWdw;    /* ptr to user def'd dialog window for this unit */
   unitptr uMetaWdw;       /* ptr to user-def'd dialog window
                              [supersedes uDefWdw for containers]     */
   unitptr uCtlWdw;        /* ptr to control window for this unit     */
   unitptr uDuplicate;     /* ptr to circuit duplicate, if exists     */
   char    *pcUserFmt;     /* ptr to format of uUserDefWdw window     */
   char    *pcLabel;       /* ptr to this unit's label                */
   char    *pcName;        /* ptr to given name of this unit's uOrig  */
   char    *pcOldName;     /* ptr to orig name of this unit's uOrig   */
   char    *pcManPage;     /* ptr to file name of manpage             */
   char    *pcCommands;    /* ptr to user def'd commands              */
   char    *pcSourceBuf1;  /* ptr to source buffer for creation ops   */ 
   char    *pcSourceBuf2;  /* ptr to source buffer for config ops     */
   unsigned int iFlags;    /* flags: available masks are I_is_selected,
                              I_store_as_ref                          */
   int     isOperand;      /* true, if icon is referenced as operand  */
   int     isOperator;     /* true, if icon is operator               */
   int     iNum;           /* nr of instances created so far          */
   int     iVarUsage;      /* 0=none 1=only @i 2=only $i, 3=both      */
   NstIconDef_t *psFun;    /* ptr to ICONDEF-interface struct         */
   void    *psScope;       /* geometry of scope rectangle for opers   */
   char    *pcLastSave;    /* ptr to file name of uOrig's latest load */
   char    *pcLastLoad;    /* ptr to file name of uOrig's latest save */
   char    *pcOrigin;      /* if from loadable folder, this holds the
                              name and the folder the unit comes from */
   char    *pcAppWindow;   /* associated application window or NULL   */
   unitptr uDesignatedPrev; /* aux field:    designated prev icon     */
   void    *pInfo;         /* info texts for this icon                */
   int     iNumVarnameDefs; 
   char    **ppcVarnameDef; /* string array holding variable defs, if any */
   unsigned int iId;       /* circuitwise unique integer id>0 for icon;
                              0, if not set                           */
   char    *apcUnused[3];  /* padding for future extensions           */
} IconVar_t;

typedef IconVar_t *IconVarptr;

typedef struct NstWireDef_{
   int nr,   /* nr of references to this wire definition               */
   DimOut,   /* dimension of destination field at which this wire ends */
   *SrcOf;   /* array of DimOut pin indexes.
                if SrcOf[i] >=0: position of src pin in src field feeding 
                                 to destination pin at position i.
                if SrcOf[i] =-1: dest pin at pos i has not yet any writer.
             */
   unitptr *WireThatFeeds; 
             /* if dest pin i has a writer pin, the connection need
                not be part of THIS wire. The wire to which a connection
                to dest pin i belongs is always given by WireThatFeeds[i].
             */ 
} NstWireDef_t;

typedef struct NstWireShape_{
   float fX,fY,fLen;  /* geometry data */
} NstWireShape_t;


typedef struct var_wire_{
   unitptr uSrcIcon, uDestIcon;
   NstWireShape_t *psWireShape;
   NstWireDef_t   *psWireDef;
   int iCol,iDestIsOut,iSrcIsOut,iDestField,iSrcField;
} var_wire_t;


extern unitptr guRootIcon;      /* icon unit representing top level circuit */
                                /* the host of this icon is the void unit
                                   guIconRoot. */
extern unitptr guCircuitRoot, guFolderRoot, guIconRoot;

/***************************************************************************
 * 
 * Important global variables and constants:
 *
 **************************************************************************/

#define MAXLABELEN  LABEL_LEN /* max len+1 of an icon's label string.
                                 = same as max NST name length */

extern NstIconWdw_t gasIconWdw[];
extern NstIconWdw_t *gpsCurrentIconWdw; /* icon window set by the most recent
                                           call to the getIconWdw-routine. */
#define MODULEWDW   gasIconWdw[0]
#define INFOWINDOW  gasIconWdw[1]
#define CIRCUITWDW  gasIconWdw[2]
#define WIREEDITWDW gasIconWdw[3]
#define MENUEWDW    gasIconWdw[4]


/* some shorthands: */
#define BUTTON(u,t,i) (Y(u,0,i)>0)  /* t is just for comment */
#define VALUE(u,t,i) (Y(u,0,i))     /* t is just for comment */

#define NST_I_ICON -1123  /* new control code for creating
                             associated unit in icon */
#define BELL 7

#define getIconVarptr(u) (vinit(u),vget(u,IconVarptr))
#define getOrigFromIcon(u) (vinit(u), (vget(u,IconVarptr))->uOrig)
        /* returns unit that is associated with icon unit u. */
#define getIconDef(u) (vinit(u),(vget(u,IconVarptr))->psFun)
        /* returns pointer to definition struct of u */
#define getIconPos(x,y,u)  ((isIconUnit(u))?(&x=Y(u,0,0), &y=Y(u,0,1),1):0)
#define getIconDefWdw(u) (vinit(u),(vget(u,IconVarptr))->uDefWdw)
#define getIconUserDefWdw(u) (getIconVarptr(u)->uUserDefWdw)
#define getIconLabel(u)  (vinit(u),(vget(u,IconVarptr))->pcLabel)


#endif /* __NEO_TYPES_H */
