/*************************************************************************

   Version 2.0 of Source file nst_math.h

   Copyright by Helge Ritter 1992

**************************************************************************
* $Log: nst_math.h,v $
* Revision 1.2  1999/05/31 19:29:31  helge
* adapted Makefile.custom + minor changes
*
* Revision 1.1  1999/05/31 17:43:06  helge
* V7.0: math, cjgr and fft now as dynamic folder math.so
*
* Revision 1.5  1997/08/30 19:31:16  helge
* has now unit serial_comp()
*
 * Revision 1.4  1997/04/14  11:03:03  helge
 * wave_gen: corrected wrong arg types in header & made simplification in *.c
 *
 * Revision 1.2  1997/03/13  10:55:06  jan
 * Dependency cleanup.
 *
* Revision 1.30  1997/01/20 13:48:00  helge
* various minor changes.
*
 * Revision 1.29  1996/08/07  14:22:53  helge
 * New NST-Version 4.8 / Neo-Version 2.8  accumulating major changes
 *
 * Revision 1.28  1995/09/30  13:18:13  helge
 * inserted some missing declarations
 *
 * Revision 1.27  1995/09/21  11:07:56  walter
 * fixed macro
 *
* Revision 1.26  1995/08/02 16:33:02  helge
* removed bug in data_store, histogram_unit
*
 * Revision 1.24  1995/07/31  09:22:54  helge
 * some minor changes.
 *
* Revision 1.22  1995/07/16 20:55:01  helge
* enhanced rpc_unit to make remote units from neo's *.NST-files
* extended nst_rpd.c, nst_rpd_add accordingly [changes disabled by
* undefing NEO in nst_rpd_add.c]
* provided 8-bit versions of some nst-units [nst_math8 etc.]
*
 * Revision 1.21  1995/04/18  21:42:42  helge
 * mainly some enhancements of the input_window unit [scrollbars, callbacks,
 * text field initializers, patch mechanism for neo windows].
 * Minor extensions of other units: switch_input/output, iterator, dq_grab;
 * A new cam_head - unit.
 *
 * Revision 1.20  1994/10/20  00:48:10  helge
 * new units for vector operations: lim_vec, bin_vec, funct_gen, ...
 *
 * Revision 1.19  1994/04/23  11:43:26  walter
 * added FieLD Macro STATistic unit, see man-page
 *
*/
#ifndef __NST_MATH_H
#define __NST_MATH_H

#define  AvXmY_out    out_0	/* [n] */
#define  AvXmY2_out   out_1	/* [1] */
#define  VarX_out     out_2	/* [1] */
#define  VarY_out     out_3	/* [1] */
#define  CorXY_out    out_4	/* [1] */
#define  NerrX_out    out_5	/* [1] */

		/* FieLD Macro STATistic unit */

#define  fldmSTAT_M_Dim		out_0	/* [n] */
#define  fldmSTAT_MS		out_1	/* [1] */
#define  fldmSTAT_VarX		out_2	/* [1] */
#define  fldmSTAT_VarY		out_3	/* [1] */
#define  fldmSTAT_CorXY		out_4	/* [1] */
#define  fldmSTAT_NRMS		out_5	/* [1] */
#define  fldmSTAT_MS_Dim	out_6	/* [n] */
#define  fldmSTAT_NRMS_Dim	out_7	/* [n] */
#define  fldmSTAT_CurrDiff_norm	out_8	/* [1] */
#define  fldmSTAT_CurrDiff_Dim	out_9	/* [n] */

/* FLOAT MACROS (Float Macro STATistic unit(unitptr u) 
 * Note diffence to FieLD macros above)
 */
#define fmSTAT_MS(u)		Y(u,1,0) /* [1] Mean Square */
#define fmSTAT_VarX(u)		Y(u,2,0) /* [1] Variance */
#define fmSTAT_VarY(u)		Y(u,3,0) /* [1] */
#define fmSTAT_CorXY(u)		Y(u,4,0) /* [1] Correlation */
#define fmSTAT_NRMS(u)		Y(u,5,0) /* [1] Normalized RMS */
#define fmSTAT_CurrDiff_norm(u) Y(u,8,0) /* [1] */

					/* obvious */
#define RADTODEG 57.29577951308232100
#define DEGTORAD  0.01745329251994330

/**************************************************************************/

extern  unittype *min_val8(int n, unittype* dest);
extern  unittype *max_val8(int n, unittype *dest);
extern  unittype *euc_len8(int n,unittype *dest);
extern  unittype *scl_prd8(int inpins, unittype *dest);

/**************************************************************************/

extern  unittype *min_val16(int n, unittype* dest);
extern  unittype *max_val16(int n, unittype* dest);
extern  unittype *euc_len16(int n,unittype *dest);
extern  unittype *scl_prd16(int inpins, unittype *dest);

/**************************************************************************/

extern  unittype *max_val(int n, unittype* dest);
extern  unitptr sum_vec(int,unitptr);
extern  unitptr const_vec(int,float*,unitptr);
extern  unitptr serial_comp(int,int,char,unitptr);
extern  unittype *max_abs(int n, unittype* dest);
extern  unitptr diffeqn_op(int,int,unitptr);
extern  unitptr funct_gen(int,float*,float*,int,unitptr);

extern  unittype *function_box(int n, int dout, ... /* float *f_0, ..., float *f_[dout-1], unittype *dest */);

extern  unittype *rnd_gen(int n, int mode, unittype *dest);
extern  unittype *rnd_vec(int n, float* min, float *max, unittype *dest);

extern  unitptr wave_gen(int T1, int T2, int T3, int mode, unitptr host);

extern  unittype *min_val(int n, unittype* dest);
extern  unittype *mul_val(int n, unittype* dest);
extern  unittype *sum_val(int n, unittype* dest);

extern  unittype *dif_vec(int n ,unittype *dest);
extern unitptr const_vec(int,float*,unitptr);
extern  unittype *sum_vec(int n,unittype *dest);

extern  unittype *mul_vec(int n,unittype *dest);

/**** cos_vec has been renamed to cos_angle. cos_vec is now a 
              different routine [defined in nst_funs.c (H.R.)] ****/

extern  unittype *cos_angle(int n,unittype *dest);
extern unitptr scale_vec(int,float,float,float,float,unitptr);

extern  unittype *out_prd(int n1, int n2, unittype *dest);
extern  unittype *out_div(int n1, int n2, unittype *dest);

extern  unittype *euc_len(int n,unittype *dest);

extern  unittype *sqr_val(int n,unittype *dest);

extern  unittype *wav_vec(int n, float *a, unittype *dest);
extern  unittype *average_series(int n, float a, unittype *dest);

extern  unittype *trf_val(int n, float (*f)(float), unittype *dest);

extern  unittype *dif_len(int n,unittype *dest);

extern  unittype *trf_vec(int n, int m, float (*f)(float), float* mat,
               float *vec, unitptr dest);

extern  unittype *trf1_vec(int n,float (*f)(float), unittype *dest);

extern  unittype *thr_vec(int n,float *threshhold, unittype *dest);
extern  unittype *threshold_vec(int n,float threshhold, unittype *dest);


/* threshholding of input vector. */

extern  unittype *scl_vec(int n,float *scale, unittype *dest);

/* scaling of input vector. */

extern  unittype *sup_vec(int n, int nvec, unittype *dest);
 
/* superposition of nvec input vectors */

extern  unittype *scl_prd(int inpins, unittype *dest);

/* scalar product of two input vectors */

extern  unittype *normalize_vec(int n, unittype *dest);
extern  unittype *sub_avg(int n, unittype *dest);

/* normalization of input vector to unit euclidean length. */
extern  unittype *bin_vec(int n, float a, float b, unittype *dest);
extern  unittype *lim_vec(int n, float a, float b, unittype *dest);

extern  unittype *min_vec(int n, unittype *dest);

extern  unittype *max_vec(int n, unittype *dest);

extern  unittype *cmp_vec(int n, int numf, float *beta, float (*f)(float), unittype *dest);
/* competitive unit with 2*numf+1 input fields, each of dimension n,
   two output fields of n pins and one output field of numf pins.
   The unit computes numf distances d_i = d(in_[i], in_[numf+i]),
   between pairs of n-dimensional vectors in[i] (vector at input field i)
   and in[i+numf] (vector at input field i+numf), i=1..numf,
   and yields three output vectors out_0, out_1 and out_2, defined
   by
         out_0 = a    (vector with numf components a_i),
         out_1 = sum_i a_i*in[i],
         out_2 = sum_i a_i*in[i+numf],

   where a_i is given by

         a_i = f(d_i)/sum_i f(d_i)     for beta != 0

             1 for d_i = min_j d_j
             0 else                    for beta = 0

   if f=NULL, exp(-.) is used as default function.
   Input field 0 may be used to specify the n components of a
   diagonal distance metric g_k, k=0..n-1, ie. d_i is computed
   according to

       d_i = sum_k g_k*(x_k - y_k)^2

   where x_k, y_k denote the k-th component of input fields
   in_[i] and in_[i+numf], respectively.
*/

extern  unittype *statistics(int n, unittype *dest);

/* creates a unit with two n-dimensional inputs x and y.
   On each call of exec, an updated value of the following
   averages appears at the 6 output fields out_0..out_5:

   <x - y>            field 0   (vector at pins 0..n-1)
   <[x - y]^2>        field 1
   <[x - <x>]^2>      field 2
   <[y - <y>]^2>      field 3
   <[x-<x>][y-<y>]>             field 4
   sqrt(<[x-y]^2>/<[y-<y>]^2>)  field 5 (normalised mrs-error)
				(for y=const, a value of -1 is returned).
*/


#endif
