/******************************************************************************
 *
 * This file contains the header for NST-unit method
 *
 * Implements fast call-by-reference of named units with auto-adapting
 * interface.
 *
 * Copyright Helge Ritter 1997
 *
 * $Source: /vol/ni/src/cvsroot/nst5/nstsrc/nst_method.h,v $
 * $Log: nst_method.h,v $
 * Revision 1.9  2005/09/02 13:57:33  rhaschke
 * cleanup in declarations
 * nst_[scrn,X11,prog]: adjustments and additions for NST7's neosrc
 *
 * Revision 1.8  2005/05/11 08:57:09  rhaschke
 * reenabled warning for unbound virtual_method on adaption
 * exported exec_use_method to easily create a "virtual use_method" without need
 * to use a use_method as orig for virtual_method
 *
 * Revision 1.7  2000/01/14 14:42:48  helge
 * bug correction: unresolved use_method instances with only inputs or only
 *    outputs failed to memorize their previous interface.
 * new function nst_ctrl_use_method_warnings() to disable/enable warnings
 *    from ctrl_use_method().
 *
 * Revision 1.6  1999/03/09 13:45:39  helge
 * Added some routines needed to correct remake bug in neo4.c
 *
 * Revision 1.5  1998/09/07 08:08:13  helge
 * added pcOption parameter to use_named_unit constructor.
 * New options %o and %O allow to change data transfer
 * policy for output data (current default: by reference)
 * by "by redirection" (%o) and by "by copy" (%O)
 * (see MANPAGE for details)
 *
 * Revision 1.4  1998/09/01 17:16:08  helge
 * added query function nst_caller_of_method()
 *
 * Revision 1.3  1998/01/05 18:16:52  helge
 * new unit virtual_method() implementing parameters of type ``method''
 * within class containers wrapping C or C++ classes.
 *
 * Revision 1.2  1997/12/21 15:50:44  helge
 * minor adjustments
 *
 * Revision 1.1  1997/12/20 23:42:37  helge
 * new use_method unit to call (C or C++) methods wrapped as NST units.
 * Similar to use_named, but faster & with automatically formed interface.
 *
 *
 *****************************************************************************/

#ifndef __NST_METHOD_H
#define __NST_METHOD_H

extern unitptr return_unit(unitptr);  /* for convenience */
extern int     nst_is_use_method(unitptr);
extern int     nst_is_virtual_method(unitptr);
extern int     nst_method_is_bound(unitptr);
extern char   *nst_get_name_of_virtual_method(unitptr);
extern int     nst_set_name_of_virtual_method(char*,unitptr);
extern unitptr use_method(char*,int,char**,int,char**,char*,unitptr);
extern int exec_use_method(unitptr);
extern unitptr virtual_method(char*,unitptr,int(*)(unitptr),
                       int(*)(unitptr,unitptr),char*,unitptr);
extern unitptr nst_root_for_ref_search(unitptr uNewRoot);
extern unitptr nst_find_referenced_unit(char*,unitptr);
extern unitptr nst_unresolved_reference(unitptr);
extern int nst_save_defs_for_class_container(FILE *fp, unitptr);
extern int nst_load_defs_for_class_container(FILE *fp, unitptr);
extern unitptr nst_caller_of_method(void);
extern unitptr nst_ref_of_method(unitptr);
extern void    nst_update_method(unitptr);
extern void nst_ctrl_use_method_warnings(int);
extern void nst_reconnect_use_method(unitptr u);
#endif

