#ifndef __NST_NEURON_H
#define __NST_NEURON_H

#define NST_FREE_WEIGHTS -101
#define NST_PRUNE_WEIGHTS -102
#define NST_RANDOMIZE_WEIGHTS -103

typedef float transfer_func_t(float);

typedef struct var_instar_{
  int iNumInp;     /* nr of incoming weights for this node */
  float *pfWeight; /* ptr to array of iNumInp values for incoming weights */
  unsigned short *piFrom; /* ptr to iNumInp indices belonging
                             to nodes sending incoming weights */
} var_instar_t;

typedef struct var_outstar_{
  /* NOTE: in order to avoid double storage of each weight, 
           the outstar_t structs only hold references to
           the weight values. The weight values themselves
           are always held in an array psInstar[] of instar_t structs.
  */
  int iNumOut;     /* nr of outgoing weights from this node */
  unsigned short *piTo;  /* ptr to iNumOut indices belonging
                            to nodes receiving outgoing weights. */
  unsigned short *piWhich;  
  /* k-th outgoing weight fWeight_k is given by

         fWeight_k = psInstar[piTo[k]].pfWeight[piWhich[k]]

     where psInstar[] is the array of the weight-holding instar_t
     structs. Note also that psInstar[piTo[k]].piFrom[piWhich[k]]=
     index position of outstar-struct from which piTo,piWhich were
     taken.
  */
  unsigned short iNextEntry;  /* aux index */
} var_outstar_t;

typedef struct var_neuron_pool_{ /* local var of unit neuron_pool: */
   int iMode,iNeurons,iCoord,iInitInput;
   var_instar_t **ppsInstar;  /* ptr to start of array of iNum var_instar els */
   float (*pActivation)(float);  /* activation function to be used */
   float (*pActivationDeriv)(float);  /* derivative of activation function to be used */
} var_neuron_pool_t;



typedef struct var_weight_pool_{ /* local var of unit weight_pool: */
   int iSrcNeurons, iDestNeurons;
   /* float (*pDistance)(int,float*,float*); */
   float afScale[3];
   float fProbScale;
   float (*pWeightProb)(float);
   float (*pWeightValue)(float);
   float (*pWeightRndGen)(void);
   /* functions to define adaptation rule: */
   int iLearningRule, iWinnerMode, iForbidSignChange;
   int iLimitWeights;
   float fWeightMin,fWeightMax;
   float (*pFF)(float);
   float (*pGG)(float);
   float (*pHH)(float);
   float (*pWeightChange)(float,float,float);
   float *pfPres, *pfPost;  /* ptrs to aux buffers */
   char  *pcOpt,*pcFile;
   int iSaveFormatted,iTotalWeights;
   var_instar_t  *psInstar;
   var_outstar_t *psOutstar;
} var_weight_pool_t;
  

typedef struct var_inspect_weight_pool_{ /* local var of unit inspect_weight_pool: */
   char *pcOpt;
} var_inspect_weight_pool_t;


extern unitptr neuron_pool(int,int*,float*,float(*)(float),float(*)(float),char*,unitptr);
extern unitptr inspect_weight_pool(int,char*,unitptr);
extern unitptr weight_pool(int,int,float*,float(*)(float),
   float(*)(float),float(*)(void),float(*)(float),float(*)(float),float(*)(void),char*,unitptr);

extern int exec_inspect_weight_pool(unitptr);
extern int adapt_inspect_weight_pool(unitptr);
extern int ctrl_inspect_weight_pool(int,unitptr);
extern int load_inspect_weight_pool(FILE*,unitptr);
extern int save_inspect_weight_pool(FILE*,unitptr);
extern int remove_inspect_weight_pool(unitptr);

extern int exec_neuron_pool(unitptr);
extern int adapt_neuron_pool(unitptr);
extern int ctrl_neuron_pool(int,unitptr);
extern int load_neuron_pool(FILE*,unitptr);
extern int save_neuron_pool(FILE*,unitptr);
extern int remove_neuron_pool(unitptr);

extern int exec_weight_pool(unitptr);
extern int adapt_weight_pool(unitptr);
extern int ctrl_weight_pool(int,unitptr);
extern int load_weight_pool(FILE*,unitptr);
extern int save_weight_pool(FILE*,unitptr);
extern int remove_weight_pool(unitptr);

extern float FUN_LOGISTIC(float);
extern float FUN_DLOGISTIC(float);
extern float FUN_REGINV(float);
extern float FUN_REGINV2(float);
extern float FUN_REGINV3(float);
extern float RNDFUN_GAUSS(void);
extern float RNDFUN_GAUSS1(void);
extern float RNDFUN_UNIFORM11(void);
extern float RNDFUN_UNIFORM01(void);
extern float RNDFUN_BINARY11(void);
extern float RNDFUN_BINARY01(void);
extern float RNDFUN_CONST1(void);
extern float FUN_BP(float, float, float);
extern float FUN_VOID(float);
extern float FUN_SIGNEDSQUARE(float);

#endif
