
/******************************************************************************
 * Purpose:
 * Usage:
 * Requires:
 * $Source: /vol/ni/src/cvsroot/nst5/nstsrc/nst_prog_aux.h,v $
 * $Log: nst_prog_aux.h,v $
 * Revision 1.44  2005/05/12 14:00:12  rhaschke
 * corrected several bugs in class assignment code and simplified this code
 * removed CompileSwitch CLASS_ELEMENTS (used as default now)
 *
 * Revision 1.43  2003/11/08 11:29:28  helge
 * added new flag iIsTransient to loadlib_t struct to make automatically
 * created loadlibs compiled prog unit functions recognizable.
 *
 * Revision 1.42  2003/10/06 13:47:31  helge
 * NEW FEATURE: preceding a function with the new keyword 'compiled' will
 *   invoke the C-compiler to re-implement this function as a compiled function
 * imported from a (invisibly) created, shared #import library. This occurs
 * in a fully transparent way, the only observable thing is a dramatic speed-up
 * (1-2 orders of magnitude!) in the execution of the function. Current main
 * restrictions are: - no calls to other #import or prog-functions - no safety
 * checks against array bounds - all compiled functions must come first.
 *   Further details: see prog_unit MANPAGE, new section "Compiled Functions".
 * BUG CORRECTIONS:
 *  - postfix operations of the type 'a[i++] = expr(i);' (i.e., postfixed
 *    variable occurred also on RHS) assigned RHS to the correct array el, but
 *    with i *in* expr() already inc'ed! Now conforms to correct C-behavior.
 *  - new feature of declaring vector elements in class instances directly
 *    as output pins ("OUT float obj.vec[];") did not work properly. Fixed.
 *
 * Revision 1.41  2003/07/15 10:20:47  helge
 * nst_prog_aux[45].c (new feature/bug correction): now return types of
 *    class valued element functions are no longer restricted to classes
 *    defined in the same loadlib (use qualified class name).
 * Error messages in the case of undefined class return types do not any
 *    longer miss the point.
 * nst_prog_aux.c nst_prog_aux[1245].c nst_prog_aux.h: (new feature):
 *  - now element variables of class instances can become OUT pins, syntax:
 *    "OUT float instance.myvec[], float instance.myvar;" or, to get dyn pin:
 *    "OUT float *instance.myvec, float instance.myvar;"
 *  - in loadlib implementation, oldstyle declaration "float *myvec" may
 *    be replaced by "float myvec[]" to indicate that myvec will never be
 *    reallocated/resized during the lifetime of the object.
 *    Consequence: myvec[] will permit both dynarray and fixed dim OUT pins,
 *    *myvec only dynarray OUT pins.
 *
 * Revision 1.40  2003/06/11 15:43:29  jontrup
 * (minor change) added declaration of nst_redimension()
 *
 * Revision 1.39  2002/11/25 08:56:00  helge
 * (bug corrections) 1. undeclared variables in the definition of a class
 *    variable caused a crash. Reason was that the automatic definition
 *    of these vars changed the next free location for storing the
 *    returned class value (only class constructors are affected).
 *    Now taken into account by using a new reserved index value
 *    YET_TO_BE_DEFINED which postpones look-up of the class index
 *    until execution time (in CallImportedFunction()).
 * 2. when class constructor in a class variable declaration raised
 *    an exception we got a crash. Now we get a syntax error (minor changes
 *    in CompClassVarDef()).
 *
 * Revision 1.38  2002/11/11 12:18:38  helge
 * (bug correction): implementation of __NAME__ was too simple-minded and
 *    caused a crash.
 *
 * Revision 1.37  2002/08/27 14:41:53  helge
 * (new feature) char *TYPEINFO_T() function to provide unique id string
 * for permit type compatibility checking of template classes including their
 * type bindings (for details, cf. tutorial & demolib8.c). Entailed to change
 * pActualTemplateArgInfo (in class_var_t) to a new typeinfo_t type.
 *
 * Revision 1.36  2002/08/20 09:11:16  helge
 * - changes needed by recent modifications
 *
 * Revision 1.35  2002/08/12 07:02:06  rhaschke
 * added declarations of exported functions
 *
 * Revision 1.34  2002/08/05 09:02:03  helge
 * - added some definitions
 *
 * Revision 1.33  2002/07/22 16:54:05  helge
 * bug correction: re-enabled old "funcid<n>" loadlib function declarations.
 *
 * Revision 1.32  2002/07/17 10:27:46  helge
 * some changes to implement templates and structs
 *
 * Revision 1.31  2002/03/06 08:49:01  helge
 * now each #import class can declare an optional CTRL_xxxx(int,objptr)
 * function to allow instances to react to a NST control message.
 *
 * Revision 1.30  2001/10/31 10:37:58  helge
 * new features for implementing #import libs:
 * - now scalar #import class member variables can be defined as readonly
 *   by prefixing their type specifier with the keyword "readonly".
 * - vector #import class member variables can be made resizeable at the prog
 *   unit level by prefixing their type specifier with the keyword "dyn".
 * - new non-opaque connectors allow access to element variables of class objects:
 *   INP class foo #f1(); OUT class foo #f2() (better doc soon);
 * - as a special case this allows to implement structs
 * - new library STD_EXTENSIONS that is always included.
 *   Can be used to implement prog unit extension functions (please
 *   use conservatively). Implements new function void sleep(secs).
 * bug corrections:
 * - system():  now trailing blanks insignificant for comparison with
 *             nst_permit_system_calls file.
 * - in some cases (&) redimension code was not carried out.
 * - several other minor corrections
 *
 * Revision 1.29  2001/09/14 14:23:42  helge
 * (bug corrections) - due to static variables in CallImportedFunction()
 * recursive calls of imported functions did not work (this bug could
 * only appear when function ptrs were used).
 * - when an exception occurred already during initialization of a variable,
 *   this could cause a crash
 * (new features) - now function pointers as arguments may also pass class
 *   member functions
 * - importlib functions can now be declared with
 *   short, short*, ushort, ushort*, double, and double* parameters
 *   and with short, ushort or double return values as well.
 *   For short, short*, ushort, ushort* one may pass
 *   int and int* parameters, for double, double* one may pass float
 *   and float* (in both cases, there will be an automatic conversion
 *   before the parameter is passed to the loadlib implementation and
 *   a back-conversion for the result).
 *   "long" for a loadlib param is now a synonym for "int" (as in C on intel)
 * - loadlib function arguments may now be defined
 *   with the optional qualifiers "in", "out", "inout" for CORBA purposes
 *   Likewise, there is a "oneway" qualifier as a return value.
 * - verbose mode now also shows function parameter names for better doc
 * - some further changes to support CORBA via new import library corba.so
 *
 * Revision 1.28  2000/10/16 17:13:56  helge
 * minor changes for compilation with solaris cc -v -Xa
 *
 * Revision 1.27  2000/09/06 06:22:32  helge
 * nst_prog_aux{1,2}.c (bug correction): vector assignments of the type
 * a := something  and  xxx(...) := something  with xxx=row,col or rect
 * made the stack grow by one float per call, causing overflow if in
 * long loops (corrections to functions EndStoreVector() and
 * EndStoreRect() in nst_prog_aux2.c and to INS_COPY_VECTOR case in
 * nst_prog_aux1.c).
 *
 * nst_prog_aux.c (bug correction) arithmetics for unbracketed factors,
 * such as a/b/c was implicitly a/(b/c) for variables, but (a/b)/c for
 * constants. Now left-to-right (a/b)/c evaluation order applies for both
 * situations and in conformance with C-usage [also for % and for * and
 * mixtures of these].
 *
 * nst_prog_{aux,aux1,aux3}.c nst_prog_ops.h: (bug correction):
 * changes in SetInstructions() and new op codes INS_LNMUL,DIV,MOD
 * to handle factor expressions with a minus sign immediately after the
 * *, / or % operator (such as  a/-b, 4*-5, c/-dimof(v) etc.) correctly.
 *
 * nst_prog_aux1.c (minor change) print_instructions(): prints some
 * more information for some op codes.
 *
 * nst_prog_aux.h (minor change) new struct element iDebugMode for
 * debugging extensions (not yet operative).
 *
 * Revision 1.26  2000/06/24 20:26:16  helge
 * some new definitions
 *
 * Revision 1.25  2000/04/28 13:54:00  helge
 * major revision of prog_unit: several bugs removed, major new features are
 * (see prog unit man page and tutorial for details):
 * 1. boolean expressions:  now evaluated as in C, i.e., as soon as the result
 *    is determined, further terms are not any more evaluated (eg., 0 && i++
 *    will no longer increment i)
 * 2. better runtime error handling: runtime errors will now "throw" exceptions,
 *    that can be selectively handled with catch-blocks, analogously as in C++
 *    (with simplifications, cf. man page). New prog unit function "throw()"
 *    throws an exception "by hand" and "catch", "errtype", "errtext" are now
 *    reserved words.
 *    NOTE: previous prog unit code with unhandled run-time errors may require
 *    insertion of  "catch {printf("%s\n",errtext);}" before return from the
 *    run-time-error-producing routine/main to produce the same behavior as
 *    previously.
 * 3. unhandled exceptions (from both prog- and other NST units) can propagate
 *    up in the NST unit hierarchy and be handled by distant error handlers
 *    implemented as suitable prog_units (cf. man page and new tutorial
 *    chapter ErrorHandling).
 * 4. name spaces to avoid collisions when loading many packages:
 *    eg. add optional definition    char *NAMESPACE_xxxx = "abc"; to
 *    wrapper of loadlib xxxx.so to distinguish (at prog_unit src level) all
 *    references to objects with optional prefix "abc". Absence of NAMESPACE
 *    def will expect library base name as prefix when name collision occurs.
 * 5. unassigned instances (omit parameter list of constructor) and resizeable
 *    arrays of class objects, e.g. class foo x, y[10];
 * 6. class/function wrapping:  declarations can specify default values for
 *    trailing float/int args, e.g.   "myfun(char*,float=3.14,int=10)",
 *    usage as in C++.
 * 7. wrapped functions can now have function parameters. Type declaration
 *    uses the highly intuitive ANSI-C scheme, e.g., define in wrapper
 *    "myfun(float(*)(int,char*))" to define a function "myfun" with a single
 *    function parameter of a function with arguments (int,char*) returning float.
 * 8. class member functions can return newly created instances (ownership
 *    must be passed to caller!):     "class myfun(...)" => x = myfun(...);
 *    yields new class instance x.
 * 9. class wrappers: type name of extern class function arguments now
 *    needs name space prefix (e.g., "f(abc.foo,int") instead of "f(foo,int)").
 * (see ImportingLibraryObjects.HLP.html for details on 5-9)
 * 10. new file nst_prog_aux5.c to avoid the others growing too large.
 *
 * Revision 1.4  2000/03/07 20:36:13  helge
 * before new indirection layer
 *
 * Revision 1.24  2000/02/23 15:22:27  helge
 * several improvements to class usage:
 *   - if a class (e.g. "foo") is declared for which so far no #import library was
 *     loaded, an automatic #import of a library "classname.so" (i.e. "foo.so")
 *     is attempted.
 *   - class arguments now also for non-member functions
 *   - classes used in parameter lists are no longer restricted to the same package
 *   - an #import library can now define derived classes that extend existing
 *     classes ("subclassing"). (see tutorial: ImportingLibraryObjects.HLP.html)
 *     example in tutorial/demolib5a,b,c.[ch]
 *   - embedding strings "classname = version" in the wrapper permits automatic
 *     consistency check for class versions
 * (bug correction) #import class member functions with void argument list
 *    were processed incorrectly (->crash)
 *
 * Revision 1.23  2000/02/07 17:08:59  helge
 * new exec_container_opnds() call to execute operands of enclosing container
 *
 * Revision 1.22  2000/01/13 18:32:24  helge
 * corrected some parsing problems after intro of overloaded functions
 *
 * Revision 1.21  2000/01/10 18:05:09  helge
 * 1. overloaded #import functions/methods
 * 2. #import functions can now be declared with return types float, int, void
 * 3. #import constants can now be declared as int or float
 * 4. #import libraries can now also be linked statically (requires to append
 *    library base name "nnnn" with underscore to FUNCTIONS/CLASSES symbol)
 * 5. INP pointer variables are no longer saved/loaded (the storage of such variables
 *    is purely a matter of the source connector!)
 * 6. "assignments" from void functions were accepted in old code, now forbidden
 * 7. _init()/_fini() functions can now be declared to get prog_unit_id parameter
 *    passed
 * 8. several bug corrections
 *
 * Revision 1.20  1999/09/27 15:08:15  helge
 * A number of changes:
 *        - BUG correction: when nr of variables exceeded limit, reallocation
 *          of internal buffers was not yet handled properly. Should now be ok.
 *         int, int[] and int* variables and function parameters.
 *        - const values have now their type (int/float) inferred
 *          from the expression that defines their value
 *        ! IMPORTANT CHANGE:
 *          arithmetics now type-sensitive (i.e., 3/5=0!=3./5., but bytes are
 *          treated like ints, i.e., no automatic mod 256 clipping for
 *          intermediate byte results, only after storing back into byte element).
 *       BACKWARD COMPATIBILITY:
 *          If environment variable PROG_UNIT_ALL_FLOAT is defined
 *          and has value 1, the new int-rounding is disabled.
 *          Changes in this variable are only noticed after a re-translation
 *          of the prog unit code [which also occurs when a prog unit is
 *          loaded from file]!
 *        - int vectors also allowed in most matrix algebra extensions
 *          (restrictions same as for bytes) BUT: for both bytes and int,
 *           arithmetics in matrix algebra expressions is currently always
 *           like for floats (except within scalar subexpressions, in which
 *           the int-rounding applies), until value is assigned
 *           back to int/byte array (then conversion occurs)
 *        - Composite multi-pin ("white") fields with {...} after INP/OUT.
 *        - vector calls for built-in, source defined, or loadlib'ed
 *          unary float functions, e.g.  pfResult := sin(pfVec1+2*pfVec2);
 *        - "static" keyword for global vars now optional; omitting it
 *          is equivalent as adding an empty "={}" - initialization,
 *          i.e., non-static declared global vars are not saved to file.
 *        - variable declarations no longer restricted to appear
 *          before function definitions.
 *        - OUT/INP char *pc  now yield string pins.
 *          Input strings work by reference, output strings by copying.
 *        - for-loop optimization was broken and is now restored.
 *        - optimized simple matrix algebra assignments such as
 *           pfTo := pfFrom;
 *          (this is now carried out without allocating the temporary buffer
 *          that is needed for more complicated expressions)
 *        - now NULL allowed in place of an array or object argument parameter
 *          (except for '-' parameter lists)
 *        - some corrections in loadlib class wrapper code
 *        - now re-instantiation of class variables possible, unless these
 *          are INP or OUT variables, example in examplesrc, file foo.C
 *        - loadlib wrappers can now specify interface functions in the
 *          usual form, e.g., instead of ".foo<.*;>" one may write
 *          "float foo(float,float*,int)" etc (see documentation for
 *           "+","-" equivalents etc.). The old notation remains still
 *           valid and both notations may even appear mixed (but not
 *           within the same function).
 *        - some safety for system command: before executed, the command is
 *          checked against a number of templates that must be specified in
 *          a write-protected .nst_permit_system_calls file. If no such
 *          file exists or no match is found, the command will not be executed.
 *        - also no execution of system() in plugin mode.
 *
 * Revision 1.2  1999/08/05 18:55:26  helge
 * TYPE_ID eliminated
 *
 * Revision 1.1  1999/08/05 18:11:44  helge
 * Initial revision
 *
 * Revision 1.19  1999/07/18 11:01:46  helge
 * (loadlib definitions for classes): now "a"-"z" new argument type
 *    indicators to define class member functions that have other class
 *    instances (of the same loadlib package) as their argument parameters.
 *
 * Revision 1.18  1999/06/22 20:02:22  helge
 * added update mechanism to make class element variables that are arrays
 * dynamically resizeable (see MANPAGE).
 *
 * Revision 1.17  1999/06/21 16:48:48  helge
 * - now byte-sized data types char and byte for prog_unit programs
 *   (no longer use of floats to represent strings), declare with
 *   char and byte keywords.
 * - NST INP/OUT: char s[] yields string, byte b[] yields byte array pin ,
 *                byte *s  yields dynamic byte pin, char* do not use yet
 * - LOAD/SAVE: bytes will be saved to full length, chars only up to first '\0'
 * - prog unit functions can now have char* (in addition to float,float*) args
 * - RESTRICTION: newly introduced string concatenation with '+' is now
 *   restricted to variables with char or byte elements (no longer floats)
 * - loadlib functions: '%' defs writeable byte* arg, ':' var number of '%'s
 *   (will be passed as ptr to char-stride spaced elements; formerly only
 *    readable char spaced '$' was available, write was limited to float)
 * - no byte or char scalars, only arrays/pointers
 * - BACKWARD COMPATIBILITY ISSUES: char s[5] is now changed to allocate
 *   an array of 5 byte sized integers instead of 5 floats (previously,
 *   char arrays were float arrays, but INP/OUT mapped them to a string pin).
 *   Old float array usage for strings is still possible but deprecated.
 *   Substituting float* or float[] identifier into loadlib function
 *   expecting char* (with char stride now!) will invoke
 *   automatic conversion behind the stage, but associated overhead may be
 *   visible, i.e, rewrite time critical portions so that types do match.
 *   Of course, now chars can no longer hold fractional values.
 * - IMPORTANT:  pointer assignments into arrays,  p = a+offset,
 *   where p and a are of different element type (char/float) are
 *   presently still tolerated, but may yield "undesireable behavior".
 *   (probably they will be forbidden soon).
 * - MINOR CHANGE to loadlib class defs: element identifiers that shall become
 *   arrays must now be prefixed by type char (* for float, % for byte arrays)
 *   Example:  myclass = {..., "*myfloatarray", "%mybytearray", ... }
 * - Matrix algebra operations work on bytes and floats as well, except
 *   left/right division, inversion and exponentiation with exponent !=0,1.
 *
 * Revision 1.16  1999/06/17 12:41:22  helge
 * some corrections to previous check-in
 * Dimension revision directive now analogous to C-cast, e.g.
 *   myfun( (float[iTmpDim]) pV )
 * revises dim of pV temporarily to iTmpDim for function call
 *
 * Revision 1.15  1999/06/16 16:02:54  helge
 * Major upgrade of prog_unit:
 * 1. some extensions to string handling (p,q,r pointers):
 *   p += "hello";       // appends
 *   p  = q + strdup(r)  // concats
 *   strdup(r,len) duplicates first len (or all, if absent) chars of r
 * 2. #loadlib's can now define class types (description => manpage)
 *   declaration as static vars, e.g. static class myclass myvar(arg1,arg2);
 *   Calls like myvar.method(4.3);  x=myvar.p[45];  etc.
 * 3. Class instances declared as
 *    INP class myclass x(2,3); OUT class myclass y(2,3);
 *    will appear as pins of user-defined type; wires allow
 *    different prog units to access objects in other prog units.
 *    y = x;  will copy x into y to transport an object across
 *    a unit. More details, see prog_unit man page and logistic.c
 *    example in examplesrc.
 * 4. new "dimension revision directive" allows to "fake" modified
 *    dimension when passing array/pointer to loadlib'ed functions:
 *    foo(pV1 (DIM=3), pV2 (DIM=5*i+1))  does what it suggests.
 *    NOTE: directive not permitted for non-loadlib functions!
 * 5. bug correction: shifts to array args (e.g. pcStr+1)
 *    passed as argument parameters to a function defined in the
 *    prog unit source code were lost when the argument was passed
 *    to a further function inside the body of the first function:
 *    foo(float *pcStr) { printf("c=%c f=%s",pcStr[0],pcStr); }
 *    pcStr = "hello";  foo(pcStr+1);  // printed "c=e f=hello"!
 *    Now corrected.
 *
 * Revision 1.14  1999/05/25 12:34:58  helge
 * 1. added new functions for use within loadlib implementations:
 *   void nst_prog_def_userobj(void *pAdr,void (*pFree)(void))
 *   void *nst_prog_userobj(void)
 * to allow allocation of private object per prog-reference
 * into a loadlib (for details, see man page).
 * 2. moved recently added SET_INFO_FOR() calls from nst_prog.c
 *   into neo-wrapper, where they actually belong.
 *
 * Revision 1.13  1999/05/05 10:22:39  helge
 * C++ style comment on same line after INP/OUT definition is now used
 * by Neo as info text for the defined input/output field of the unit.
 *
 * Revision 1.12  1999/02/14 17:36:02  helge
 * - corrected four bugs:
 *   bug in CallFunction(): Local pointer vars as function arguments came
 *   with wrong offsets:
 *                   void fun1(float *p) { p[0]=1; }
 *                   void fun2(void) { float *q=new(10); fun1(q); }
 *   caused out-of-range index error when body of fun1() was executed.
 * - bug in ReturnFromFunction(): gpiCurrentIndexShift array was not properly
 *   reset to previous context after return from a function (caused
 *   out-of-range index errors for pointer function args).
 * - when an OUT pin was a dyn float that was resized during a use_method
 *   call, a memory inconsistency occurred (bug in nst_init_interface0()).
 * - vector assignment pfVec := value; could overwrite last function
 *   argument when inside the scope of a function.
 * - minor modification to macros in nst_prog_aux.h for use as left values
 * - new prog_unit directives #noimplicit and #implicit "abc" to
 *   enforce variable declarations.
 *
 * Revision 1.11  1998/10/26 16:58:00  helge
 * changes to allow creation of dynamic input/output pins via float pointers.
 *
 * Revision 1.10  1998/10/06 10:37:13  helge
 * added matrix algebra capabilities to prog_unit
 * exec_opnd() .. init_opnd() now also accepts name as argument
 * aux files now split into three (grew too large)
 * new header nst_prog_ops.h contains instruction codes
 *
 * Revision 1.9  1998/09/07 08:17:25  helge
 * some final changes to prog unit
 *
 * Revision 1.8  1998/09/01 16:02:12  helge
 * major upgrade of prog_unit: now with function definitions, pointer
 *    variables for dynamic memory allocation, more flexible interface
 *    def within code with INP/OUT keywords, void(*)(void) functions
 *    defined as "public" turn prog unit into class container with
 *    these functions appearing as method subunits.
 *
 * Revision 1.7  1997/08/30 19:45:38  helge
 * prog_unit can now use strings: are just (float!) arrays holding char values.
 * Initialization:  static s[]="hello";
 * Two new functions:  strlen(s) and system(s).
 *   #loadlib "string.so"  for more string functions (see manpage)
 *   #loadlib "file.so"  for file access functions (fp's have become floats, too!)
 *
 * Revision 1.5  1997/07/22  11:31:08  helge
 *  o prog_unit can now have one-dimensional static arrays and const values:
 *    declare and optionally initialize as usual at the beginning, i.e.
 *   const n=2,pi=4*atan(1); static float a[2*n]={1,2,3,4},b[]={0.5,1+sqrt(2)};
 *
 * o a new #loadlib "dlibname.so" directive allows dynamic extension
 *   by single- and multi-argument functions in self-written libraries.
 *   (see new man page for details)
 *
 * o an example is #loadlib "solid.so" that makes a number of GL and
 *   NST-rendering functions callable from the prog_unit
 *
 * Revision 1.4  1997/04/02  14:10:02  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.3  1997/03/21  12:49:22  jan
 * Minor cleanup.
 *
 * Revision 1.2  1997/03/13 10:55:45  jan
 * Dependency cleanup.
 *
 * Revision 1.7  1997/01/20 13:48:52  helge
 * various minor changes.
 *
 * Revision 1.6  1996/08/07  14:23:22  helge
 * New NST-Version 4.8 / Neo-Version 2.8  accumulating major changes
 *
 * Revision 1.5  1995/09/22  21:07:15  helge
 * version can now be run under X11 when NST_X11 is defined. YGL no longer
 * needed. Requires new packages nst_X11.c nst_aux5.c and header glX11.h.
 *
 * Revision 1.1  1995/03/06  13:34:06  helge
 * auxiliary routines for nst_prog.c
 *
 *****************************************************************************/
#ifndef __NST_PROG_AUX_H
#define __NST_PROG_AUX_H

#include <stdio.h>
#include <stddef.h>

#include "nst_types.h"

#define MAX_FUN_PTRS 10  /* max nr of function ptrs provided per slot */  
#define TEMPLATES   /* enable template processing */
#define STRUCTARGS  /* allow structs as function args */
#define STATICARG  

#define SHARED_LIB_SUFFIX ".so"
/* #define ARRAYARGCHAR "*&%$" */
#define ARRAYARGCHAR "/*&%$"
#define SCALARARGCHAR ".;"  /* is for src function args */
#define STRUCT_TYPELETTER0   'a'
#define STRUCT_TYPELETTER1   'z'
#define IS_STRUCT_ARGCHAR(C) (STRUCT_TYPELETTER0 <= (C) && (C) <= STRUCT_TYPELETTER1)
#define NAME_OF_UNDEFINED_CLASS "__UNDEFINED_CLASS__"
#define VAR_OF_UNDEFINED_CLASS  "$VAR_OF_UNDEFINED_CLASS"
#define LIB_OF_UNDEFINED_CLASS  NAME_OF_UNDEFINED_CLASS
#define LIB_OF_ERROR_HANDLING   "error_handling"
#define STD_EXTENSIONS          "std_extensions"
#define NAMESPACE_FOR_STRUCTS   "struct"


/* macros to set NST Pin addresses before executing : */

#define MAXVAR 100
#define TEMPLATE_MAXARGS 10      /* max nr of template type parameters */
#define NUM_BUILTIN_FUNCTIONS 30

#define TYPE_CONST_VARIABLE  -1  /* any negative val */
#define TYPE_SCALAR_VARIABLE 0   /* don't change! */
#define TYPE_ARRAY_VARIABLE  1   /* any positive val */ 
#define TYPE_CONST_STRING    2   /* one larger */
#define TYPE_CLASS           IS_CLASS 

#define IMPOSSIBLE_INDEX  999999.5   /* any large non-integer that is not yet rounded */
#define HUGE_ARRAY_INDEX  999999999  /* any unreasonably large index value */
#define REFERENCE_CAST    ((float)100000000)  /* another unreasonably large index value,
                                                 rounded to a float */
#define ADDED_FUN_MAXARGS 50         /* max nr of args for added functions */

#define VOID_FUNCTION     0
#define FLOAT_FUNCTION    1
#define INT_FUNCTION      2
#define PUBLIC_FUNCTION   3
#define UNTYPED_FUNCTION  4 
#define VECTOR_FUNCTION   5 

#define IS_DYN_VAR        1
#define IS_OBJ_VAR        2

#define RETURN_VALUE          1
#define RETURN_VOID           2
#define RETURN_VECTOR         3 
#define RETURN_NO_VECTOR      4 
#define RETURN_VOID_OR_VECTOR 5 


/**************** ref count debugging stuff ********************/
 
#ifdef DEBUGREFS

typedef struct ref__t {
  int iRefs;      /* nr of refs. MUST ALWAYS BE IN 1st position!! */
  int iCapacity;  /* max nr of ref (inc/dec) events */
  int iNumEvents; /* actual nr of inc/dec events */
  int *piEvent;   /* array to iCapacity locations (first iNumEvent locs used):
                     each location holds a line number that identifies where
                     the event took place ( - = dec + = inc event) */
  char *pcEvent;  /* array to iCapacity chars for ref op codes */
} ref_t;

extern void add_ref_info(char,ref_t*);
#else 
typedef int ref_t;
#endif

extern void print_ref(ref_t *pRef);


/***** basic instruction type: *********************************/

typedef struct Instruction_{
   int iType;
   int iMaxIndex;
   union {
      float fConst;                   
	  ptrdiff_t iAdr;
	  ptrdiff_t iOffset;
	  int   iArgs;
	  int   iLabel;
	  int   iFunct;
	  char  *pcArgType;
	  size_t iInstruc;
      void  *pFunctionDef;
   } v;	 
#ifdef TEMPLATES
   char *pcTypeOfArgForTemplate;  
                     /* for non-templates: NULL.
                        for templates: holds pcTypeOfArg specially made for
                        the instance of this call
                     */
#endif
   short int iContext;
   short int iLine; 
} Instruction_t;


typedef Instruction_t* Instructionptr;

typedef float *(*ptrfunction_t)(int*,void*);
typedef float **(*dynptrfunction_t)(int**,void*);
typedef float **(*initfunction_t)(int**,void*);

/* these numbers need not be the same! : */
#define MAXFINIS         20  /* max nr of registereable _fini functions per unit */
#define MAXMETHODS       20  /* max nr of methods in a single program */
#define METHOD_MAXFIELDS 20  /* max nr of input/output fields a method can have */
#define STRUCT_MAXVARS   25  /* max nr of elements in a struct */

#define NORMAL_ITERATOR   1
#define VIRTUAL_ITERATOR  2

typedef struct prog_method_{
   struct function_ *pFunction;     /* this methods function */
   char *pcName;                    /* print name of this method */
   int iNumInp, iNumOut;            /* nr of inputs and outputs  */
   int *piDimOfInp, *piDimOfOut;    /* their dimensions          */ 
   int *piAdrOfInp, *piAdrOfOut;    /* their symbolic adress     */ 
   int *piPinOfInp, *piPinOfOut;    /* the pin numbers           */
   int iInpFields, iOutFields;      /* nr of NST inp and out fields */
   char **ppcTypeOfInp, 
        **ppcTypeOfOut;             /* their type strings (s=scalar, 
                                       f=float t=text string)    */
   int *piTypeOfInp, *piTypeOfOut;  /* their TYPE_MASKed type numbers 
                                       IS_FLOAT_VARIABLE .. IS_CLASS
                                    */
   float ***pppfInp, ***pppfOut;    /* pppfInp[f] = location where 
                                       adress *(pppfInp[f]) of first el 
                                       of f-th input variable is held.
                                       Analogously pppfOut.  */
   /* additional info for input fields: */
   struct class_def 
          **ppClassDefOfInp;        /* class def ptr of each input or NULL */
   int     *piInpVarDefs;
   int    ***pppiInpVarDim0;       /* adr of adr where dim of var of field f is held */
   float ****ppppfInpVarDef0;      /* adr of adr where adr of var of field f is held */
   float *pfBuf;                   /* buffer for scalar float inputs */
   int   *piBuf;                   /* buffer for scalar int inputs */
   unsigned char *pcBuf;           /* buffer for scalar char inputs */
   struct class_var_ **ppClassVar; 
   char **ppcInfoOfInp, **ppcInfoOfOut;  /* info strings for fields */
   void *u;                         /* the NST method subunit */
   int iIsIterator;                 /* flag */
} prog_method_t;

typedef struct prog_data_{

   /*** public interface to input and output variables/methods: */

   int iNumMethods;            /* total nr of exported methods */
   prog_method_t **ppMethod;   /* ptr array describing installed methods */
   int (*method)(int);         /* element function to invoke methods */
   char *pcName;               /* ptr to string name of program. 
                                  References externally held NST string to
                                  track renamings. DONT FREE */
   unitptr uProgUnit;          /* the prog unit itself
                                  [available only AFTER compilation!]
                                */

   /*** everything below this line may become changed! Don't use! */

   /*** private data structures for compiled program: */

   Instruction_t *pInstruc;   /* array with program instructions
                                 (NULL, if compilation unsuccessful) */
   /*** to save current environment during callbacks (such as exec_opnd() 
        with other prog_units!): */

   char *pcDiagnostics;       /* diagnostics string in case of unsuccessful
                                 compilation */ 
   char *pcSource;            /* source string in case of #debug option */
   int  iDebugMode;           /* for debugging */ 
   int  iSkipBreaks;          /* for debugging */
   struct static_var_  
            *pStatic;         /* ptr to struct holding all static variables */
   int     iNumTypes;         /* nr of data types defined */
   struct type_ **ppType;     /* storage block of type descriptions */

   int     iNumFunctions;     /* nr of source functions defined */
   struct function_ **ppFunction; /* storage block of function descriptions
                                 (hold local variables in turn) */

   /*** info about dynamically loaded extensions: */
   int iImportsAreFinalized;  /* flag */
   int iNumLoadlibs;          /* nr of dynamically loaded libraries */
   struct loadlib_ 
   **ppLoadlib;               /* ptr to array of iNumLoadlibs loadlib
                                     descriptions */
   char **ppcVisibleFunctions;  
   char **ppcVisibleClasses;  
   char **ppcVisibleConstants;  
   /* for each loadlib, a mask string of iNumFunctions/iNumClassDefs
                                 chars indicating
                                 for each function/class in the loadlib its
                                 visibility status:
                                 '+'  =  this element is visible by its 
                                         short name
                                 '!'  =  this element is visible by its 
                                         fully qualified name [not yet used]
                                 any other: this element iselement is invisible,
                               */
   int  iNumImportedFunctions;
   char **ppcImportedFunction;
   int  iNumImportedClasses;
   int  iExtraImportedStructDefs;  /* the last iExtraImportedStructDefs entries
                                      of the iNumImportedClasses entries in
                                      ppcImportedClass[], ppImportedClassDef[]
                                      are struct definitions from the source code
                                      [cf. AddStructDefToImport() in prog_aux5.c]
                                   */
   char **ppcImportedClass;
   int  iNumImportedConstants;
   char **ppcImportedConstant;
   struct function_def_ **ppImportedFunctionDef;  
                               /* lookup array for all imported
                                  and visible non-member functions */
   struct class_def **ppImportedClassDef;  
                               /* lookup array for all imported
                                  and visible non-member functions */
   int *piImportedConstDef; 
   struct loadlib_ **ppImportedConstDefLib; 
 
   void
   **ppUserObjAdr;               /* ptr to array of iNumLoadlibs user 
                                     obj pointers */
   int iIsExceptionHandler;      /* if 1, this prog is an exception handler */
   int iReturn, iIndexOfErrStr;
   Instruction_t *pErrInstruction;

   /*** for use within NST: */

   int iId;                   /* a unique int identifier for 
                                 the associated prog_unit   */
   /* fields to indicate interface mapping of special arrays X,Y,Z,T: */
   char acType[5];            /* type info for X,Y,Z,T:
                                 'i'  input from scalar field
                                 'I'  input from packed field
                                 'o'  output to scalar field
                                 'O'  output to packed field   */
#ifdef MAX_FUN_PTRS
   int aiFunctionForFunPtrId[MAX_FUN_PTRS];
   int aiClassIndexForFunPtrId[MAX_FUN_PTRS];
#endif

} prog_data_t;

extern Instruction_t *CompileInstructions(char *pStr, char **ppcError);    
extern void FreeVarStorage(float **ppfVars); 
extern float ExecuteInstructions(int iLabel, prog_data_t* pProgData);
extern void print_instructions(int,Instructionptr pExp);    
void free_prog_data(prog_data_t *pD);

/*** some global pointers, set for the macros below: */

extern float **gppfStaticVar, ***gpppfStaticVar, **gppfAutoVar;
extern int   *gpiStaticType,  *gpiAutoType;
extern int   *gpiStaticDim, **gppiStaticDim,  *gpiAutoDim;

#define DOT_REPLACER 'O'  /* char to replace dot in obj.name tokens to make C-identifier */
#define REPLACE_DOT(S,D)  if ((D=strchr(S,'.'))) *D=DOT_REPLACER
#define RESTORE_DOT(D)    if (D) *D='.'

#define HIDDEN_UNIT_ARRAY "$1$"  /* name of a hidden 1d array with el=1 */
#define ADR_OF_NULL  (-1) /* adr of reserved static var that represents NULL */

/**** some macro defs: */

/* to be used at run-time: */
#define ADR_OF_VAR(i)      (((i)<0) ? gpppfStaticVar[i][0]  : gppfAutoVar[i])
#define ADR_OF_STATIC_VAR(i) (gpppfStaticVar[i][0])
#define CLASS_VAR_OF(I)    ((class_var_t*) ADR_OF_VAR(I))
#define ADR_OF_BYTE_VAR(i) ((unsigned char*)((((i)<0) ? gpppfStaticVar[i][0] : gppfAutoVar[i])))
#define ADR_OF_INT_VAR(i)  ((int*)((((i)<0) ? gpppfStaticVar[i][0] : gppfAutoVar[i])))
#define ADR_LOC_OF_VAR(i)  (((i)<0) ? gpppfStaticVar[i] : gppfAutoVar+(i))
#define ADR_LOC_OF_OBJ(i)  gpppfStaticVar[i]
#define VAR(I)             (((I)<0) ? gpppfStaticVar[I][0] : gppfAutoVar[I])[0]
#define INT_VAR(I)         ((int*)(((I)<0) ? gpppfStaticVar[I][0]  : gppfAutoVar[I]))[0]
#define SVAR(I)            gpppfStaticVar[I][0][0]
#define AVAR(I)            gppfAutoVar[I][0]

/* to be used at compile-time: */
#define ARRAY_VAR(I,S)    ((((I)<0) ? gpppfStaticVar[I][0]    : gppfAutoVar[I])+S)[0]
#define BYTE_ARRAY_VAR(I,S) (((unsigned char*)(((I)<0) ? gpppfStaticVar[I][0]  : gppfAutoVar[I]))+S)[0]
#define INT_ARRAY_VAR(I,S) (((int*)(((I)<0) ? gpppfStaticVar[I][0]  : gppfAutoVar[I]))+S)[0]
#define TYPE_OF_VAR(i)     (((i)<0)  ? gpiStaticType     : gpiAutoType)[i]
#define STRIDE_OF_VAR(i)   (TYPE_OF_VAR(i) & ARRAY_STRIDE_MASK)
#define TYPE_LOC_OF_VAR(i) (((i)<0) ? gpiStaticType+(i) : gpiAutoType+(i))
#define TYPE_LOC_OF_OBJ(i) (gpiStaticType+(i))
#define DIM_OF_VAR(i)      (((i)<0) ? gppiStaticDim[i] : gpiAutoDim+(i))[0]
#define DIM_OF_STATIC_VAR(i) gppiStaticDim[i][0]
#define DIM_LOC_OF_VAR(i)  (((i)<0) ? gppiStaticDim[i] : gpiAutoDim+(i))
#define DIM_LOC_OF_OBJ(i)  gppiStaticDim[i]
#define TYPE_CHAR_OF(i)    (gpProgData->ppType[i]->cTypeChar)

/*** the next is true whenever variable i references dynamically 
     allocated memory outside of the normal blocks: */
#define ACTIVE_POINTER(i) (TYPE_OF_VAR(i) & IS_ACTIVE_POINTER)

typedef struct type_{
  /* description of a data type used by the prog_unit */
  char *pcName;                 /* the name of the data type */
  char cTypeChar;               /* a one letter short typename */
  void *(*pCopy)(void*,void*);  /* its copy constructor */
  void (*pFree)(void*);         /* its destructor */
  /* further methods may be added in the future */
} type_t;

typedef struct symbol_{
  int iNum,     /* nr of symbols in use */
  iMaxNum,      /* max nr of entries allowed   */
  *piSym;       /* piSym[i] = int index to represent
                   symbol name ppcName[i] after parsing
                */
  char
  **ppcName;    /* array of iMaxNum ptrs to symbol
                   names (first iNum used) */
} symbol_t;

typedef struct static_var_{
   int iNumVars,         /* nr of entries of ppfStaticVar and piDim */
   iMaxVars,             /* max nr of entries allowed in ppfVar */
   iUsed,                /* nr of float positions used in pfVar */
   iMaxFloats;           /* current limit on nr of floats
                            (reallocate pfVar when needing more) */
   float
   **ppfStaticVar;       /* array of ptrs into pfVar,
                            access goes through this array, using
                            *negative* indices */
   float ***pppfStaticVar;
   int *piDim;           /* array for dimension info: */
   int **ppiDim;         /* array of ptrs into piDim */
   int *piType;          /* array of type numbers  */
   float *pfVar;         /* start adress of storage block */
   symbol_t  *pSym;      /* ptr to static symbol table */
   int iGlobalSaveCount; /* nr of global symbols to save */
   int iTotalSaveCount;  /* total nr of symbols to save */
} static_var_t;

typedef struct function_{
    char *pcName;         /* text name of this function                   */
    char *pcTypeOfArg;    
                          /*  string encoding argument types of this 
                              function: '.' = float, '*' = float*.        */
    int  iFunctionType;   /*  one of VOID_FUNCTION, FLOAT_FUNCTION or 
                              INT_FUNCTION or PUBLIC_FUNCTION             */
    prog_method_t *pMethod;  /* if the function is a public function
                                (= method), its interface info is here!   */
    int  iNumArgVar;       /* the nr of argument parameters               */
    int  iNumArgAutoVar;   /* nr of automatic var that are args           */
    int  iNumArrayArg;     /* nr of array (including char*) arguments     */
    int  iNumCharArrayArg; /* nr of char* array arguments                 */
    int  iNumStructArg;    /* nr of struct arguments                      */
    int  iNumTotalVar;     /* total nr of vars (including struct args)    */
#ifdef STATICARG
    int  iFirstStaticArg;  /* index of first static parameter */
#endif
  symbol_t 
    *pStatic, *pAuto;     /* ptrs to symbol_t structs describing 
                             this functions static and autom
                             variables (pAuto includes arg params!) 
                          */
    float *pfVar;         /* an initialized set of values for all 
                             non-parameter automatic variables 
                             of this function (for rapid initialization
                             of the same field in each context record
                             for this function). */
    ptrdiff_t *piVar;     /* piVar[i] = address offset of i-th nonparameter
                             value w.r.t. to pfVar. Corresponding indices
                             by iNumArgs lower than in context_t ppfVar[]!
                          */
    int iAuxBytes;        /* the byte size required jointly by the arrays
                             piDim, piTypei, piShift and ppfAutoVar 
                             (the last twor only in the context_t struct). 
                             The value is iNumAutoVar*(sizeof(float)
                               +2*sizeof(int)) +iNumArgVar*sizeof(int)
                          */
    int iUsedBytes;       /* used byte size of the pfVar block */
    int iMaxFloats;       /* max nr of float els storable in pfVar block */
#ifdef AUTOPLUS
    int **ppiDim;           /* dimension and type info for all non-parameter 
                             automatic variables */
#else
    int *piDim;           /* dimension and type info for all non-parameter 
                             automatic variables */
#endif
    int *piType;          /* array of type numbers */
#ifdef STRUCTARGS
    int iMaxTotalVar;      /* max # of auto var dims that fit in piDim */
#else
    int iMaxAutoVar;      /* max # of auto var dims that fit in piDim */
#endif
    int  iStart, iLast;   /* offset to first and last instruction         */
    int iLocalSaveCount;  /* nr of local static symbols to save */
} function_t;

#define FAST_HANDLED_STRUCTARGS 4   /* for 4 struct args or less we provide
                                       static ppiRefs buffer for significant
                                       faster handling */

typedef struct context_{
   function_t *pFunction;  /* ptr to function_t struct of invoked function */
#ifdef AUTOPLUS
   float ***pppfAutoVar;    /* array of iNum pointers to the adr of the initial
                              element of each automatic variable
                              (includes parameter and non-parameter vars) */
#else
   float **ppfAutoVar;     /* array of iNum pointers to the initial
                              element of each automatic variable
                              (includes parameter and non-parameter vars) */
#endif
   int *piDimAndTypeBak;   /* to restore Function's dim and type array at return.
                              Current values always written into pFunction */
   int *piShift;           /* index shift for all parameter auto variables
                              (accumulates offsets from prior calls) */
   float *pfVar;           /* storage block for iNumNonArg automatic 
                              variables that are not argument parameters */
   int iNumArgBufs;        /* number of temporarily created argument buffers
                              for those array arguments that need type conversion */
   void **ppvArgBuf;       /* iNumArgBufs pointers to tmp created arg buffers */
   void **ppvArgOri;       /* iNumArgBufs pointers to adr where to write buf back */
   int  *piArgDim;         /* iNumArgBufs dims (=nr of els in buffer) */
   int  **ppiRefs;         /* array of ptr to iNumStructArgs ref cnt variables 
                              belonging to struct arguments. After return, the 
                              ref count to each element must be decremented.  */
   int *apiRefs[FAST_HANDLED_STRUCTARGS]; /* for use by ppiRefs */ 
   struct context_ *prev;  /* pointer to context record of
                              enclosing call, or NULL, if
                              function call was at outer level. */
   Instruction_t *pInstruc; /* the instruction to continue with after return.*/
   float *pfStack;
} context_t;

#define IS_CONSTRUCTOR_FUNCTION(F) ((F)->iPassHandle == 2)
#define IS_CLASS_FUNCTION(F) ((F)->iPassHandle == 1)

typedef struct function_def_{
   char *pcName;        /* string name of function */
   char *pcSymName;     /* string name as used by dlsym */
   struct class_def 
   *pClassDef;          /* the class def to which this function belongs */
   int  iArity;         /* nr of allowed args */
   int  iArrayArgs;     /* the nr of array args among them */
   int  iClassArgs;     /* the nr of class args among them */
   int  iOffset;        /* >0, if this is a class var. In this case value at pfAdr[iOffset] */
   char *pcTypeOfArg;   
                        /* indicates expected argument types:
                           each letter stands for one or several arguments:
                           '0..9'  float args, plain passing via float
                           '.'  single float arg, passing via float** array
                           '?'  arbitrary nr of float args, passing via float*
                           '*'  single array arg, passing via float**
                           The letter specifies also implicitly, how
                           the parameters are passed to the library
                           function
                           '$'  array used as text string, passed
                           via float**, but not writeable
                           (to simulate const char*)
                           only one of these can come last:
                           '+'  one or more trailing writeable array arg (*)
                           '-'  one or more trailing readable args ($ or .)
                        */
   char *pcIDLTypeOfArg; /*  the same, but with additional interspersed
                             IDL parameter in/out status indicator chars */
   int iIDLQualifier;   /* to indicate further IDL qualifiers to a function 
                             IDL parameter in/out status indicator chars */
   struct arginfo_  
   *pArgInfo;           /* additional info about each function argument */
   int iPassHandle;     /* indicates use of function call with additional arg */
   unsigned int iReturnType;  /* type specification as given by TYPE_OF_VAR() */
#ifdef TEMPLATES
   int iTemplateReturnType; /* a nonzero value (one of A_T0..A_T9) 
                      indicates that the function returns a template type */
#endif
   char *pcClassReturnType;  /* the string name of the class ret type, if any */
   float *(*pVarAdr)(int*,void*); /* if !NULL: return variable adress */
   float **(*pDynVarAdr)(int**,void*); /* if !NULL: return variable adress */
   /* float **(*pInitVar)(int**,void*); */ /*  */
   union {
      float (*fv0)(int,float*);       /* for type '?': pass args by float*  */
      float (*fv1)(int*,float**); /* for types '.','*','$': pass args by float** */
      float (*fv2)(int,int*,float**); /* if '+','-' admits var nr of trailing args */
      int   (*iv1)(int*,float**); /* for types '.','*','$': pass args by float** */
      int   (*iv2)(int,int*,float**); /* if '+','-' admits var nr of trailing args */
      void  (*vv1)(int*,float**); /* for types '.','*','$': pass args by float** */
      void  (*vv2)(int,int*,float**); /* if '+','-' admits var nr of trailing args */
      unsigned char (*bv0)(int,float*);       /* for type '?': pass args by float*  */
      unsigned char (*bv1)(int*,float**); /* for types '.','*','$': pass args by float** */
      unsigned char (*bv2)(int,int*,float**); /* if '+','-' admits var nr of trailing args */
      char (*cv0)(int,float*);       /* for type '?': pass args by float*  */
      char (*cv1)(int*,float**); /* for types '.','*','$': pass args by float** */
      char (*cv2)(int,int*,float**); /* if '+','-' admits var nr of trailing args */
      short (*sv0)(int,float*);       /* for type '?': pass args by float*  */
      short (*sv1)(int*,float**); /* for types '.','*','$': pass args by float** */
      short (*sv2)(int,int*,float**); /* if '+','-' admits var nr of trailing args */
      unsigned short (*uv0)(int,float*);       /* for type '?': pass args by float*  */
      unsigned short (*uv1)(int*,float**); /* for types '.','*','$': pass args by float** */
      unsigned short (*uv2)(int,int*,float**); /* if '+','-' admits var nr of trailing args */
      double (*dv0)(int,float*);       /* for type '?': pass args by float*  */
      double (*dv1)(int*,float**); /* for types '.','*','$': pass args by float** */
      double (*dv2)(int,int*,float**); /* if '+','-' admits var nr of trailing args */
      /* the remaining are for plain float passing: */
      float (*f0)(void);
      float (*f1)(float);
      float (*f2)(float,float);
      float (*f3)(float,float,float);
      float (*f4)(float,float,float,float);
      float (*f5)(float,float,float,float,float);
      float (*f6)(float,float,float,float,float,float);
      float (*f7)(float,float,float,float,float,float,float);
      float (*f8)(float,float,float,float,float,float,float,float);
      float (*f9)(float,float,float,float,float,float,float,float,float);
      /* if iPassHandle: use functions with an objptr as additional last arg: */
      float (*Fv0)(int,float*,void*);   /* for type '?': pass args by float*  */
      float (*Fv1)(int*,float**,void*); /* for types '.','*','$': pass args by float** */
      float (*Fv2)(int,int*,float**,void*); /* if '+','-' admits var nr of trailing args */
      int   (*Iv1)(int*,float**,void*); /* for types '.','*','$': pass args by float** */
      int   (*Iv2)(int,int*,float**,void*); /* if '+','-' admits var nr of trailing args */
      void  (*Vv1)(int*,float**,void*); /* for types '.','*','$': pass args by float** */
      void  (*Vv2)(int,int*,float**,void*); /* if '+','-' admits var nr of trailing args */
      unsigned char (*Bv0)(int,float*,void*);   /* for type '?': pass args by float*  */
      unsigned char (*Bv1)(int*,float**,void*); /* for types '.','*','$': pass args by float** */
      unsigned char (*Bv2)(int,int*,float**,void*); /* if '+','-' admits var nr of trailing args */
      char (*Cv0)(int,float*,void*);   /* for type '?': pass args by float*  */
      char (*Cv1)(int*,float**,void*); /* for types '.','*','$': pass args by float** */
      char (*Cv2)(int,int*,float**,void*); /* if '+','-' admits var nr of trailing args */
      short (*Sv0)(int,float*,void*);   /* for type '?': pass args by float*  */
      short (*Sv1)(int*,float**,void*); /* for types '.','*','$': pass args by float** */
      short (*Sv2)(int,int*,float**,void*); /* if '+','-' admits var nr of trailing args */
      unsigned short (*Uv0)(int,float*,void*);   /* for type '?': pass args by float*  */
      unsigned short (*Uv1)(int*,float**,void*); /* for types '.','*','$': pass args by float** */
      unsigned short (*Uv2)(int,int*,float**,void*); /* if '+','-' admits var nr of trailing args */
      double (*Dv0)(int,float*,void*);   /* for type '?': pass args by float*  */
      double (*Dv1)(int*,float**,void*); /* for types '.','*','$': pass args by float** */
      double (*Dv2)(int,int*,float**,void*); /* if '+','-' admits var nr of trailing args */
      /* the remaining are for plain float passing: */
      float (*F0)(void*);
      float (*F1)(float,void*);
      float (*F2)(float,float,void*);
      float (*F3)(float,float,float,void*);
      float (*F4)(float,float,float,float,void*);
      float (*F5)(float,float,float,float,float,void*);
      float (*F6)(float,float,float,float,float,float,void*);
      float (*F7)(float,float,float,float,float,float,float,void*);
      float (*F8)(float,float,float,float,float,float,float,float,void*);
      float (*F9)(float,float,float,float,float,float,float,float,float,void*);
      /* these are for constructors: */
      void *(*COv0)(int,float*);   /* for type '?': pass args by float*  */
      void *(*COv1)(int*,float**); /* for types '.','*','$': pass args by float** */
      void *(*COv2)(int,int*,float**); /* if '+','-' admits var nr of trailing args */
      /* these are for object constructors with additional class handle arg: */
      void *(*Ov1)(int*,float**,void*); /* for types '.','*','$': pass args by float** */
      void *(*Ov2)(int,int*,float**,void*); /* if '+','-' admits var nr of trailing args */
      /* the remaining are for plain void *passing: */
      void *(*C0)(void);
      void *(*C1)(float);
      void *(*C2)(float,float);
      void *(*C3)(float,float,float);
      void *(*C4)(float,float,float,float);
      void *(*C5)(float,float,float,float,float);
      void *(*C6)(float,float,float,float,float,float);
      void *(*C7)(float,float,float,float,float,float,float);
      void *(*C8)(float,float,float,float,float,float,float,float);
      void *(*C9)(float,float,float,float,float,float,float,float,float);
   } fun;
} function_def_t;

#define MAXTYPELEN 100
#define TYPEINFO
 
/* only for template classes: */
 
typedef struct typeinfo__t {
   char acDesc[MAXTYPELEN+1];  /* buffer for type desc */
   /* NOTE: we intentionally use an array instead of a
      ptr to permit casting a typeinfo_t to a char* that
      then yields the acDesc string adr (simple interface)!
   */
   int iNumTemplateArgs, iNonUniqueByName;
   struct class_def *pClassDef;  /* the template class def */  
   struct class_def **ppClassDef; /* the class defs of the 
                                     iNumTemplateArgs template args */   
   struct struct_def__t **ppStructDef; /* the struct defs of the 
                                     iNumTemplateArgs template args */   
   int *piTRefs;  /* adr of ref count variable */
} typeinfo_t;           

                                                          
typedef struct arginfo_{
   /* struct to keep info about an argument of a loaded function: */
   int iType;                        /* type of this argument */
   int iQualifier;                   /* to transport info about presence of CORBA
                                        IDL "in", "inout" or "out" qualifier.
                                        Has one of the three values T_input, T_output,
                                        T_inout or T_none
                                     */
   char *pcSignature;       /* a non-NULL value indicates a function argument.
                                           The string then holds the function signature
                                           [with char in pos 0 the return value] 
                                         */
   int iNumTemplateArgs;             /* for a function parameter that uses template types
                                        for its arguments, the nr of such arguments.
                                        0 otherwise.
                                      */
   struct class_def **ppFunctionClassArgDef;    
   char **ppcFunctionClassArgId;    
   char *pcClassId;    
   float fDefault;                   /* default value for float/int args */
   char *pcArgName;                  /* when the definition specified a dummy arg
                                        name, then this is here (only for more
                                        informative function printing in verbose mode)
                                     */
   struct class_def *pClassDef;      /* NULL or ptr to full class def. Points to
                                        memory NOT owned by this struct */
} arginfo_t;

typedef struct struct_def__t {
   char *pcName;     /* the name of the struct type */
   int iNumVarDefs;  /* the number of element vars */
   char **ppcVar;    /* string array giving the struct's iNumVarDefs variable names. */
   int *piType;      /* for each variable, its type */
   int *piDim;       /* for each variable, its dimension */
   float **ppfInit;  /* for each variable, an array of float initializers */
   int iAllowCopyMethod;  /* to allow copy veto by any non-copyable class member */
   struct class_var__t 
       **ppClassVar; /* to allow class instance variables as members */
   char cStructArgChar;  /* single letter code to identify struct type in function arg lists */ 
   int *piDefRefs;   /* always non-NULL: ptr to nr of struct references to this definition */
   int iImportedByProgUnit; /* if 1, the def is currently imported by a prog unit and may
                               then not become freed, even in the absence of any references */
} struct_def_t;

typedef struct class_def{
   /* description of a class: */
   char *pcClass;                    /* the class name */
   char *pcParentClass;              /* the parent class name or NULL */
   char *pcParentLibName;            /* the parent library name or NULL */
   int  iAllSuperClassesInstalled;   /* flag */
   struct class_def *pParentClass;
   struct loadlib_ *pLoadlib;        /* the loadlib to which this class belongs */
   int iNumFunctionDefs;
   function_def_t **ppFunctionDef;   /* function def for this class,  
                                       first el is always constructor */
   int iNumConstDefs;                /* nr of defined constants */
   char **ppcConstDef;               /* their names */
   float *pfConstDef;                /* their values */ 
   int   *piConstType;               /* their types */ 
   int iNumVarDefs;                  /* nr of defined non-dyn variables */
   int iNumDynVarDefs;               /* nr of defined dyn variables */
   int iNumAllVarDefs;               /* nr of defined variables */
   int iNumArrays;                   /* nr of non-dyn array variables */
   ptrfunction_t *ppVarDef;          /* array of function pointers to fetch */ 
   dynptrfunction_t *ppDynVarDef;    /* array of function pointers to fetch  
                                       address and dimension */
   unsigned char *pcIsDynVar;        /* indicator array */
   void (*pFreeClassObj)(void*);     /* destructor method for class obj */
   void *(*pCopy)(void*,void*);      /* copy method for class var */
   int  (*pCompatible)(void*,void*); /* compatibility test function for class var */
   int  (*pCompare)(void*,void*);    /* comparison function for class var */
   void (*pSave)(FILE*,void*);       /* save method for class var */
   void (*pLoad)(FILE*,void*);       /* load method for class var */
   void (*pCtrl)(int,void*);         /* ctrl method for class var */
   void (*pRestore)(FILE*,void**);   /* restore method for class var
                                        [if defined, supersedes load method] */
   char *(*pDescription)(void*);     /* description method for class var */
   void (*pUpdateFunction)(void);    /* update function for class var */
   int *piResizeIndicatorVar;        /* adr of variable to indicate whether resize is OK */
   int iResponsibleForTypeDef;       /* if this class has registered a NST data type */
   unsigned int iNonOpaque;          /* flag */
   int iNumTemplateArgs;             /* >0 if class is a template class. Max = 10 */
   int iConstructorUsesTemplateArgs; /* constructor uses template args */
   char acTemplateArg[TEMPLATE_MAXARGS+1]; /* for each template parameter its type char */
   arginfo_t *pTemplateArgInfo; /* the associated arginfos */
   struct_def_t *pStructDef;         /* in case of a struct, its definition */
} class_def_t;

#define T_input     '>'
#define T_output    '<'
#define T_inout     '-'
#define T_none      ' '
#define T_oneway    '^'
#define T_dyn       '~'
#define T_readonly  '='

#define ALLOW_NONOPAQUE  1
#define FORBID_NONOPAQUE (-1)

#define A_null         '0'
#define A_char         ''
#define A_char_vec     ''   /* not necessarily null-terminated */
#define A_byte_vec     '%'
#define A_string       '$'   /* null-terminated and meant as read-only */
#define A_byte         ''
#define A_short        ''
#define A_short_vec    ''
#define A_ushort       ''
#define A_ushort_vec   ''
#define A_int          ';'
#define A_int_vec      '&'
#define A_long         ''
#define A_long_vec     ''
#define A_float        '.'
#define A_float_vec    '*'
#define A_double       '|'
#define A_double_vec   '/'
#define A_function     '^'
#define A_object       '@'
#define A_extern_object ''

/* for template parameters: */
#ifdef TEMPLATES
#define TEMPLATEARG ""  /* these must be consecutive! */
#else 
#define TEMPLATEARG ""
#endif
/* !!! when changing these, must also replace isdigit() tests!!! */
#define A_Struct       199 
#define A_undefined    '`'

/* ten contiguous chars that are not yet used for other purposes: 
   [using ISO_8859-1, these are ''..''] */

#define A_T0           ''     
#define A_T1           '' 
#define A_T2           '' 
#define A_T3           '' 
#define A_T4           '' 
#define A_T5           '' 
#define A_T6           '' 
#define A_T7           '' 
#define A_T8           '' 
#define A_T9           ''     

/* NOTE: DO NOT USE the hex values: char consts are not
         guaranteed to be unsigned, hex values would imply unsigned!
*/

typedef struct loadlib_{
   /* description of a loadlib: */
   int iNumClassDefs;           /* nr of classes in this loadlib (including
                                   the 0-entry for functions) */
   char *pcClassArgChar;        /* iNumClassDefs shorthand chars to indicate
                                   arguments that are references to a class */
   class_def_t **ppClassDef;    /* their definitions */
   int iRefs;                   /* nr of references to this loadlib */
   int iRefsFromDerivedClasses; /* nr of references from derived classes of 
                                   other loadlibs */
   void *pDlibHandle;           /* handle to shared library */
   char *pcLibFullName;         /* full library name */ 
   char *pcLibName;             /* library base name */ 
   char *pcNameSpace;           /* optional namespace string */
   /* arrays holding version info: */
   char *pcVersionInfo;         /* version info string. */ 
   int  iNumImports;
   int  iNumVersionEntries;
   char **ppcClass;             /* names of classes that have version info */
   char **ppcVersion;           /* the associated version strings */
   int  *piIsImport;            /* flag = 1, if version is for imported class */
   /* end of version stuff */
   void (*pFreeUserObj)(void*); /* destructor method for user obj */
   void (*pFini)(void);         /* fini-function of this loadlib */
   void (*pInit)(void);         /* init-function of this loadlib */
   int  iInitArgs, iFiniArgs;   /* nr of int args (0 or 1) of init/fini function */
   int  iIsTransient;           /* flag to indicate transient loadlib created
                                   for compiled prog unit functions. Such loadlibs
                                   are never shared between different prog_units */
   struct loadlib_ *next, *prev;
} loadlib_t;

/* if a variable is of type class, then its pVar entry points to an instance
   of this data type: */

typedef struct class_var__t{
   class_def_t *pClassDef;     /* associated class definition */
   void *pClassObj;            /* private data obj for instance, or
                                  NULL, if not yet instantiated
										  (i.e., the "instance" is only a reference) 
									   */
   ref_t *piRefs;              /* ptr to ref count for pClassObj 
                                  [including this one]
                                */
   int iDim;                   /* nr of array elements (0=no array) */
   int iIsResizeable;          /* flag */
   int iAllowCopyMethod;       /* flag */
   int iForbidReinstantiation; /* flag */
   unsigned int iNonOpaque;    /* flag */
   int iAdrOf1stEl;            /* lookup index of first element variable,
                                  subsequent el vars at consecutive lower
                                  index positions iAdr-1,-2,... */
   int iCurrentEl;             /* currently used array element */
   struct class_var_array *pArray;
/* if instance belongs to template class (for each template type arg one el): */
#ifdef TYPEINFO
   typeinfo_t *pActualTemplateArgInfo;  /* descr of the supplied class types */
#else
   arginfo_t *pActualTemplateArgInfo;  /* descr of the supplied class types */
#endif
/* actually, the above could be shared among many class_var_t's, but multiple
   copies is simpler. The memory penalty for this is very very modest. */
} class_var_t;

typedef struct arg_wrap__t{
   void *pClassObj;  /* ptr of class obj */
   class_def_t *pClassDef;  /* ptr to class def to access methods */
   int iIsArray;     /* flag and index: if > 0, iIsArray-1 is the array index */
   ref_t *piRefs;      /* ptr to associated refcount variable */
} arg_wrap_t;


typedef struct class_var_array{
   void        *pClassObj;     /* ptr to class object */
   ref_t       *piRefs;        /* nr of refs to this array el */
   void       **ppvAdr;        /* adr of iNumVarDef element vars */
   int          *piDim;        /* and their dimensions */
   void       ***pppvAdr;      /* pointers to the locations of above array */
   int          **ppiDim;      /* dito (used for dyn-var element variables */
} class_var_array_t;

extern function_t *gpCurrentFunction;
extern context_t  *gpCurrentContext;
extern int        giNumFunctions;
extern static_var_t      *gpStatic;
extern symbol_t          *gpGlobal;

#define UNDEFINED_SYMBOL 9999990  /* must be positive and large! */
#define YET_TO_BE_DEFINED  (-9999990) /* must be negative and large! */
#define ILLEGAL_REDEFINITION 9999991  /* hardly this many in use */
#define ILLEGAL_SCOPE        9999992  /* hardly this many in use */

#define TYPE_ID(A)  (A & TYPE_MASK)

#define IS_FLOAT_VARIABLE      0U  /* the default must be in pos 0! */
#define IS_SCALAR_VARIABLE     (1U<<1)
#define IS_INT_VARIABLE        (1U<<2)
#define IS_CHAR_VARIABLE       (1U<<3)  
#define IS_BYTE_VARIABLE       (1U<<4)
#define IS_CONST_STRING        (1U<<5)
#define IS_CONST_FLOAT         (1U<<6)
#define IS_CONST_INT           (1U<<7)
#define IS_SHORT_VARIABLE      (1U<<8)  
#define IS_USHORT_VARIABLE     (1U<<9)  
#define IS_DOUBLE_VARIABLE     (1U<<10)  
#define IS_CLASS               (1U<<11)  
#define TYPE_MASK              (IS_AUTO_VARIABLE-1)
#define IS_AUTO_VARIABLE       (1U<<12) 
#define IS_INPUT_VARIABLE      (1U<<13)
#define IS_OUTPUT_VARIABLE     (1U<<14)
#define IS_ARRAY_VARIABLE      (1U<<15)
#define IS_POINTER_VARIABLE    (1U<<16)    
#define IS_ACTIVE_POINTER      (1U<<17)
#define HAS_FLOAT_STRIDE       (1U<<18) 
#define HAS_INT_STRIDE         (1U<<19)
#define HAS_BYTE_STRIDE        (1U<<20)
#define ARRAY_STRIDE_MASK      (HAS_FLOAT_STRIDE|HAS_INT_STRIDE|HAS_BYTE_STRIDE)
#define HAS_CONST_DIM          (1U<<21)
#define IS_EXTERN_VARIABLE     (1U<<22)
#define IS_EXTERN_REFERENCE    (1U<<23)
#define START_COMPOSITE_FIELD  (1U<<24)
#define IS_INITIALIZED         (1U<<25)
#define IS_UNDEFINED_TYPE      (1U<<26)
#define IS_SHARED_VARIABLE     (1U<<27)
#define RESIZEABLE_ELEMENT_VAR (1U<<28)
#define IS_READ_ONLY           (1U<<29)
#define IS_STRUCT_MEMBER       (1U<<30)
#define IS_FUNCTION_ARGUMENT   (1U<<31)

extern int giVerbose;
extern int definition_of_scalar_variable(char*,int*,function_t*);
extern float *explicit_auto_define(char*,int,unsigned int,int*,function_t*);
extern float *explicit_static_define(char*,int,unsigned int,int*,function_t*);
extern int index_of_symbol(char*,function_t*);
extern unsigned char *str_from_float_vec(float*, int);

extern ref_t* inc_obj_refs(ref_t*);

extern int nst_redimension(void**, int);

#endif /* __NST_PROG_AUX_H */
