/*************************************************************************

   nst_prog_ops.h   instruction codes for prog_unit
                    and some global constants

*************************************************************************/

/* some predefined prog err types: */

#define BADARG_VALUE       "badarg:value"
#define BADARG_ARRAY       "badarg:array"
#define BADARG_ARRAY_TYPE  "badarg:array:type"
#define BADARG_VALUE_NONE  "badarg:value:none"
#define BADARG_ARRAY_DIM   "badarg:array:dim"
#define BADARG_OBJECT      "badarg:object"

#define BAD_OP             "badop"
#define BADOP_DIV0         "badop:div0"
#define BADOP_INDEX        "badop:index"
#define BADOP_ARRAY        "badop:array"
#define BADOP_POINTER      "badop:pointer"
#define BADOP_ARRAY_DIM    "badop:array:dim"
#define BADOP_ARRAY_TYPE   "badop:array:type"
#define BADOP_OBJECT       "badop:object"

#define BADRES_ARRAY       "badres:array"
#define BADRES_SINGULAR    "badres:singular"

/* control ops: */

#define INS_END     0
#define INS_NOP     1 
#define INS_JMP     2
#define INS_JZ      3
#define INS_JNE   218 
#define INS_JEQ   219 
#define INS_JNES  291 
#define INS_JEQS  292 
#define INS_JNZ   293 
#define INS_LJZ     4
#define INS_LJNZ    5 
#define INS_INV     6 
#define INS_POP     7 
#define INS_DUP     8 
#define INS_BREAK   9 
#define INS_CONT   10 
#define INS_RET    11 
#define INS_CASE   12 
#define INS_POP_VEC 19 

/* 13..18 taken below! */

/* binary ops: */

#define INS_ADD    20 
#define INS_SUB    21 
#define INS_MUL    22 
#define INS_DIV    23 
#define INS_POW    24 
#define INS_ADDE   25 
#define INS_SUBE   26 
#define INS_MULE   27 
#define INS_DIVE   28 
#define INS_ADDEI  29 
#define INS_DIVEI  30 
#define INS_MULEI  31 
#define INS_SUBEI  32

/* special versions for argument arrays: */

#define INS_ADDEI1 33 
#define INS_DIVEI1 34 
#define INS_MULEI1 35 
#define INS_SUBEI1 36

#define INS_LDINCI1  13  
#define INS_LDDECI1  14 
#define INS_INCLDI1  15  
#define INS_DECLDI1  16 
#define INS_STINCI1  17 
#define INS_STDECI1  18 
 
#define INS_DIV1        37  /* as DIV, but operands reversed */
#define INS_MOD         38   /* as DIV, but operands reversed */
#define INS_CONST_FLOAT 39 
#define INS_CONST_INT   40 

/* boolean ops: */

#define INS_LT     41
#define INS_GT     42
#define INS_LE     43
#define INS_GE     44
#define INS_EQ     45
#define INS_NE     46
#define INS_NOT    47
#define INS_AND    48
#define INS_OR     49

/* unary ops: */

#define INS_LOADS    50 
#define INS_LOADA    51 
#define INS_STOR     52
#define INS_LOADI    53 
#define INS_STORI    54
#define INS_STORI1   55
#define INS_LOADI1   56
#define INS_NEG      57 
#define INS_LOAD     59 

#define INS_INC      60 
#define INS_DEC      61 
#define INS_INCI     62
#define INS_DECI     63
#define INS_LDINCI   64
#define INS_LDDECI   65
#define INS_INCLDI   66
#define INS_DECLDI   67
#define INS_STINCI   68
#define INS_STDECI   69

/* math functions: */

#define INS_ABS    70 
#define INS_SQRT   71 
#define INS_STEP   72 
#define INS_NONEG  73
#define INS_TORAD  74
#define INS_TODEG  75
#define INS_PI     76
#define INS_ERF    77 
#define INS_ERFC   78 
#define INS_SGN    79 
#define INS_EXP    80 
#define INS_LOG    81 
#define INS_LOG10  82 
#define INS_RND01  83
#define INS_GRND   84
#define INS_SEED   85
#define INS_RINT   86
#define INS_FLOOR  87
#define INS_CEIL   88
#define INS_ISNAN  89
#define INS_SIN    90 
#define INS_COS    91 
#define INS_TAN    92
#define INS_COT    93
#define INS_ASIN   94 
#define INS_ACOS   95 
#define INS_ATAN   96
#define INS_ACOT   97

/* special functions */

#define INS_SINH   98 
#define INS_COSH   99 
#define INS_TANH   100
#define INS_COTH   101
#define INS_ASINH  102
#define INS_ACOSH  103
#define INS_ATANH  104
#define INS_ACOTH  105
#define INS_FERMI  106
#define INS_BGNOF  107
#define INS_ENDOF  108
#define INS_DIMOF  109
#define INS_DIM_FROM_MASK 286 

/* pointer operations: */

#define INS_PNEW   110
#define INS_PRES   111
#define INS_PRES0  124 
#define INS_PNULL  112
#define INS_P2A    113
#define INS_PASN   114
#define INS_P2A2   139 

/* special operations: */

/* the following 7 instructions must
   have consecutive code nr's and be
   in the given order:
*/
 
#define INS_EXEC    115
#define INS_ADAPT   116
#define INS_RESET   117
#define INS_INIT    118
#define INS_CTRL    119
#define INS_OPER    120
#define INS_VIRTUAL_OPER       121
#define INS_ADAPT_OPER         122 
#define INS_ADAPT_VIRTUAL_OPER 123 

#define INS_MOVEA   125
#define INS_MOVES   126
#define INS_MOVEI   127
#define INS_CALL    128
#define INS_RETURN  129

/* composed instructions for optimization: */

#define INS_LADD  130
#define INS_LSUB  131
#define INS_LMUL  132
#define INS_LDIV1 133
#define INS_LMOD  134
#define INS_LPOW  135

#define INS_LNMUL  283
#define INS_LNDIV1 284
#define INS_LNMOD  285
/* some functions with a single string parameter: */

#define INS_STRLEN 136
#define INS_SYSTEM 137
#define INS_FUN    138

/* special combinations of jumps and inc/dec ops
   for fast for-loops: */

#define INS_JILT    140  /* inc var & jmp if less */
#define INS_JILE    141  /* inc var & jmp if leq  */
#define INS_JDGT    142  /* dec var & jmp if gt   */
#define INS_JDGE    143  /* dec var & jmp if ge   */

#define INS_ADD_VEC        144
#define INS_MULT_ADD_VEC   145
#define INS_LDIV_ADD_VEC   146
#define INS_RDIV_ADD_VEC   147
#define INS_ADD_SCALAR     148
#define INS_VECTOR         149
#define INS_POP_FAC2       150
#define INS_POW_VEC        151
#define INS_ABS_VEC        152
#define INS_TRSP_MATRIX    153
#define INS_LOAD_VEC       154
#define INS_LOAD_COL       155     
#define INS_LOAD_ROW       156     
#define INS_LOAD_RECT      157     

#define INS_BGN_STOR_VECT  158
#define INS_BGN_STOR_VECT0 159
#define INS_END_STOR_VECT  160

#define INS_BGN_STOR_ROW   161     
#define INS_BGN_STOR_ROW0  162

#define INS_BGN_STOR_COL   163     
#define INS_BGN_STOR_COL0  164

#define INS_BGN_STOR_RECT  165     
#define INS_BGN_STOR_RECT0 166
#define INS_END_STOR_RECT  167     

#define INS_APPLY_FUNCTION 168     

#define INS_BRK_POINT      169

#define INS_BGN_STR_SUM    170
#define INS_ADD_STR        171
#define INS_END_STR_SUM    172
#define INS_END_STR_ADD    173
#define INS_BGN_STR_DUP    174
#define INS_ADD_STR_DUP    175
#define INS_ADD_STR_FUN    176
 
#define INS_ARRAY          177  /* new instruction for array param
                                  puts adr & dim on stack */
#define INS_ARRAY0         178  /* array + no dim rev + offset 0 */
/* indirect operations for byte arrays: */

#define INS_ADDEI_BYTE    179 
#define INS_DIVEI_BYTE    180 
#define INS_MULEI_BYTE    181 
#define INS_SUBEI_BYTE    182 
#define INS_ADDEI1_BYTE   183 
#define INS_DIVEI1_BYTE   184 
#define INS_MULEI1_BYTE   185 
#define INS_SUBEI1_BYTE   186

#define INS_LDINCI1_BYTE  187  
#define INS_LDDECI1_BYTE  188 
#define INS_INCLDI1_BYTE  189  
#define INS_DECLDI1_BYTE  190 
#define INS_STINCI1_BYTE  191 
#define INS_STDECI1_BYTE  192 
 
#define INS_LOADS_BYTE    193 
#define INS_LOADA_BYTE    194 
#define INS_LOADI_BYTE    195 
#define INS_STORI_BYTE    196 
#define INS_STORI1_BYTE   197 
#define INS_LOADI1_BYTE   198 

#define INS_INCI_BYTE     199 
#define INS_DECI_BYTE     200 
#define INS_LDINCI_BYTE   201 
#define INS_LDDECI_BYTE   202 
#define INS_INCLDI_BYTE   203 
#define INS_DECLDI_BYTE   204 
#define INS_STINCI_BYTE   205 
#define INS_STDECI_BYTE   206 

#define INS_PRINTF  207 
#define INS_SPRINTF 208 
#define INS_ARGEND  209 

/* operations for int vars: */

#define INS_LOAD_INT     210
#define INS_STOR_INT     211
#define INS_ADDE_INT     212 
#define INS_DIVE_INT     213 
#define INS_MULE_INT     214 
#define INS_SUBE_INT     215 
#define INS_DIV1_INT     216 
#define INS_MOD_INT      217 

/* indirect operations for int arrays: */

#define INS_ADDEI_INT    220 
#define INS_DIVEI_INT    221 
#define INS_MULEI_INT    222 
#define INS_SUBEI_INT    223 
#define INS_ADDEI1_INT   224 
#define INS_DIVEI1_INT   225 
#define INS_MULEI1_INT   226 
#define INS_SUBEI1_INT   227

#define INS_LDINCI1_INT  228  
#define INS_LDDECI1_INT  229 
#define INS_INCLDI1_INT  230  
#define INS_DECLDI1_INT  231 
#define INS_STINCI1_INT  232 
#define INS_STDECI1_INT  233 

#define INS_LOADS_INT    234 
#define INS_LOADA_INT    235 
#define INS_LOADI_INT    236 
#define INS_STORI_INT    237 
#define INS_STORI1_INT   238 
#define INS_LOADI1_INT   239 

#define INS_INCI_INT     240 
#define INS_DECI_INT     241 
#define INS_LDINCI_INT   242 
#define INS_LDDECI_INT   243 
#define INS_INCLDI_INT   244 
#define INS_DECLDI_INT   245 
#define INS_STINCI_INT   246 
#define INS_STDECI_INT   247 

#define INS_INC_INT      248
#define INS_DEC_INT      249

#define INS_JILT_INT     250  /* inc var & jmp if less */
#define INS_JILE_INT     251  /* inc var & jmp if leq  */
#define INS_JDGT_INT     252  /* dec var & jmp if gt   */
#define INS_JDGE_INT     253  /* dec var & jmp if ge   */

#define INS_COPY_VECTOR     254
#define INS_APPLY_FUNCTION1 255
#define INS_APPLY_FUNCTION2 256
#define INS_APPLY_FUNCTION3 257
#define INS_SRCFUNC0        258
#define INS_SRCFUNC1        259

#define INS_PASS_FUNCTION   260
#define INS_CLASS_FREE      261
#define INS_CLASS_FREE3     262
#define INS_CLASS_NEW       263
#define INS_CLASS_CTOR      264
#define INS_ASN_CLASS       265
#define INS_ASN0_CLASS      266 
#define INS_ASN1_CLASS      267 
#define INS_ASN2_CLASS      268 
#define INS_ASN3_CLASS      269 

#define INS_CPY_CLASS       270 
#define INS_CPY0_CLASS      271 
#define INS_CPY1_CLASS      272 
#define INS_CPY2_CLASS      273 
#define INS_USE_OBJ         274
#define INS_CHECK           275
#define INS_RES_OBJ         276
#define INS_DIMOF1          277 
#define INS_THROW           278 
#define INS_CATCH           279 

/*          already taken:  283 */
/*          already taken:  284 */
/*          already taken:  285 */
/*          already taken:  286 */

#define INS_BGN_VEC_TO_SCALAR0 287
#define INS_BGN_VEC_TO_SCALAR  288
#define INS_END_VEC_TO_SCALAR  289 
#define INS_SYNC_PORTS         290 
#define INS_BGN_STOR_SUBVECT  294 
#define INS_BGN_STOR_SUBVECT0 295 
#define INS_END_STOR_SUBVECT  296 
#define INS_EXISTS            297 
#define INS_EXISTS1           298 
/* the next must always be the highest three:
   [cf. nst_prog_aux1.c, allows to do three tests in one] */
#define INS_BACKSTO           299 
#define INS_BACKSTO_INT       300 
#define INS_BACKSTO_BYTE      301 
#define INS_TEMPLATE_NEW      303
#define INS_TEMPLATE_NEW3     305
#define INS_PASS_OBJ          306
#define INS_CLASS_SET_NULL    307
#define INS_CLASS_SET_NULL2   308
#define INS_COMMENT           309
#define INS_NAME_STR          310

#define INS_INC2              311   /* postfix inc */
#define INS_DEC2              312   /* postfix dec */
#define INS_INC2_INT          313   /* postfix inc */
#define INS_DEC2_INT          314   /* postfix dec */
#define INS_HIGHEST           320   /* no fixed instruction code higher than this */
