/*****************************************************************************
 *
 * nst_strg.h  -  header for NST string handling functions
 *
 * Copyright Helge Ritter, 1995
 *
 * $Source: /vol/ni/src/cvsroot/nst5/nstsrc/nst_strg.h,v $
 *
 * $Log: nst_strg.h,v $
 * Revision 1.12  2005/10/31 14:40:20  rhaschke
 * added Nst[Inp|Out]StrCat() declarations
 *
 * Revision 1.11  2005/09/02 13:57:33  rhaschke
 * cleanup in declarations
 * nst_[scrn,X11,prog]: adjustments and additions for NST7's neosrc
 *
 * Revision 1.10  2005/07/28 09:12:21  rhaschke
 * corrected analyze_printf_fmt (last change allowd writing to const memory)
 *
 * Revision 1.9  2004/03/17 12:32:26  rhaschke
 * changes for replacement of va_list's in printf_unit
 *
 * Revision 1.8  1998/01/20 18:44:09  helge
 * new routine nst_explicit_newlines() to transform newline into substr "\n"
 *
 * Revision 1.7  1997/12/21 00:04:56  helge
 * several minor changes
 *
 * Revision 1.6  1997/08/30 19:51:06  helge
 * added NstPinStrRealloc function
 *
 * Revision 1.5  1997/04/15  15:38:31  helge
 * some changes to compile nst on alpha arch [compile with gcc]
 *
 * Revision 1.4  1997/04/02 14:11:07  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.3  1997/04/02 07:00:01  helge
 * intermediate check-in
 *
 * Revision 1.1  1997/03/11  07:42:20  jan
 * NST sources for NEO version 5.0
 * Revision 1.2  1997/03/13  10:56:16  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST_STRG_H
#define __NST_STRG_H
#include <stdarg.h>
#include "nst1.h"
#include "nst3.h"

extern void remove_trailing_chars(char *pcStr, char *pcRemoveThese);
extern char *strdup_without_verts(char*);
extern void nst_process_quoted_chars_of(char*);
extern char *nst_explicit_newlines(char*);
extern int analyze_printf_fmt(char*,int,char*,char**,char**,
                              char**,int*,int*,int,va_list,char***);
extern int analyze_scanf_fmt(int,char*,char**,char**);
extern char *expand_scan_fmt_str(char *pcFmt);
extern void NstPinStrRealloc(pin *pPin, int iLen);

extern char *NstStrCpy(char*,char*);
extern char *NstStrCat(char*,char*);
extern char *NstStrCat1(char *pcDest, char *pcStr, int iOffset);
extern char *NstPinStrCpy(pin*,char*);
extern char *NstInpStrCpy(unitptr,int,int,char*);
extern char *NstOutStrCpy(unitptr,int,int,char*);
extern char *NstInpStrCat(unitptr,int,int,char*);
extern char *NstOutStrCat(unitptr,int,int,char*);

/* same routines, but admitting additional offset: */

extern char *NstStrCpy1(char*,char*,int);
extern char *NstStrCat1(char*,char*,int);
extern char *NstPinStrCpy1(pin*,char*,int);
extern char *NstInpStrCpy1(unitptr,int,int,char*,int);
extern char *NstOutStrCpy1(unitptr,int,int,char*,int);
extern char *NstInpStrCat1(unitptr,int,int,char*,int);
extern char *NstOutStrCat1(unitptr,int,int,char*,int);

extern char* NstSprintf(char*,char*,...);
extern char* NstVSprintf(char*,char*,va_list);

extern char* NstOutPrintf(unitptr,int,int,char*,...);
extern char* NstOutVPrintf(unitptr,int,int,char*,va_list);

#endif /* __NST_STRG_H */
