#ifndef __NST_TCL_SCRIPT_H_
#define __NST_TCL_SCRIPT_H_

#define EXEC_OPND   0
#define ADAPT_OPND  1 
#define INIT_OPND   2
#define RESET_OPND  3 
#define EXEC_OPNDS  4

#define NST_TCL_MAP   -270
#define NST_TCL_UNMAP -271
/* same def also in nst_aux5.c! */

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "nst.h"

typedef struct tcl_method_{
   char *pcName;                    /* print name of this method */
   int iNumInp, iNumOut;            /* nr of inputs and outputs */
   int *piDimOfInp, *piDimOfOut;    /* their dimensions */
   char *pcTypeOfInp, *pcTypeOfOut; /* their types (s=scalar, f=float
                                       t=text string) */
   int *piTypeOfInp, *piTypeOfOut; /* their type numbers (0=float,
                                      1=scalar,2=text string, ...)
                                    */
   int *piNumInpNames,
       *piNumOutNames;              /* nr of names in ppcInp/OutName */
   char ***pppcInpName;             /* pppcInpName[f][i] = name of */ 
   char ***pppcOutName;             /* i-th variable of input field f,
                                       pppcOutName[f][i] analogously */
   unitptr *u;                      /* the NST method subunit */
   int iIsIterator;                 /* flag */
} tcl_method_t;

extern void free_tcl_method_t(tcl_method_t*);

typedef struct var_tcl_prog_{ /* local var of unit tcl_script: */
  int iNumMethods, iCurrentMethod;
  tcl_method_t **ppMethod;
  char *pcProg;            /* the tcl program */
  char *pcInitialization;  /* its initialization part */
  Tcl_Interp *pInterp;
  Tk_Window  pWindow;
  int iWindowExists;
} var_tcl_prog_t;

extern void tcl_add_named_area_command(unitptr);
extern void tcl_do_initialization_events(void);
extern void tcl_enable_backingstore(const char*);
extern void tcl_map_with_children(const char*,int);
extern void tcl_set_var(var_tcl_prog_t*);
extern void try_main_event_loop(void);
extern void tcl_load_window_status(FILE*,char*);
extern void tcl_save_window_status(FILE*,const char*);
extern int init_tcl_interpreter(var_tcl_prog_t*);
extern void tcl_set_nst_instruction(void (*)(int,int,char**,var_tcl_prog_t*));
extern unitptr nst_tcl_prog(char*,unitptr);
extern int num_tcl_methods_of(var_tcl_prog_t*);
extern char *name_of_tcl_method(int,var_tcl_prog_t*);
extern int num_tcl_inputs_of(int,var_tcl_prog_t*);
extern int num_tcl_outputs_of(int,var_tcl_prog_t*);
extern char **nst_tcl_inpdesc_of(int,var_tcl_prog_t*);
extern char **nst_tcl_outdesc_of(int,var_tcl_prog_t*);
extern void do_initialization_events(void);
extern void try_main_event_loop(void);
#endif
