#ifndef _TOPOLOGY_H_
#define _TOPOLOGY_H_

#include <stdio.h>
#include <stdarg.h>

#ifndef CALLOC
#define CALLOC(A,B) (B*) calloc((A),sizeof(B))
#endif

// we separate the hyperbolic and the SOM stuff in two base classes:

class topology{
   int iDim;
   int *piSize;
 protected:
   // these MUST BE SET BY THE INHERITING CLASS!
   int iOrder;   // max. order of a node
   int iNodes;   // nr of nodes.  
   class node **ppBaseNode;  
 public:
   int get_num_nodes(void) { return iNodes; }  // get nr of nodes
   int get_dim(void) { return iDim; }          // get dimension
   int *get_size(void) { return piSize; }      // get size info
   int get_index(float *);                     // get index from position
   void *for_nodes(void*,int*);                // iterate over all nodes
   void *for_edges(void*,int,int,int*,int*);   // iterate over all edges
   void prepare_neighborhood(int,int);
   void for_neighborhood(int,int,
           void*,void(*)(int,void*));          // apply function to region
   // these must be implemented:
   virtual float *get_pos(int)=0;              // get position vector of a node
   virtual float dist0(float*,int)=0;          // distance between point and a node    
   virtual float dist1(int,int)=0;             // lattice distance between two nodes    
   virtual float dist(float*,float*)=0;        // distance between two points
   virtual void translate(float*, float*)=0;   // translation
   virtual void rotate(float*, float*)=0;      // rotation 
   virtual int create_search_range(int**);     // create search range array & ret dim
   topology(int iDimension) { iDim = iDimension; }
};

class node {  
   // the base class to make a general node:
public:
   int iNumNb;        // nr of lattice neigbors
   node **ppNb;       // pointers to neighbors
   int iVisited;      // flag
   int iIndex;        // this node's serial index
   node(int,int);
   ~node(void); 
};

#endif 

